@@ -2,12 +2,16 @@ import os
 import copy
 import subprocess
 from math import pi, sqrt
+import pathlib
+from typing import Union, Optional, List, Set, Dict, Any
+import warnings
 
 import numpy as np
 
-from ase.utils import jsonable
-from ase.dft.kpoints import monkhorst_pack
 from ase.cell import Cell
+from ase.dft.kpoints import monkhorst_pack
+from ase.outputs import Properties, all_outputs
+from ase.utils import jsonable
 
 
 class CalculatorError(RuntimeError):
@@ -82,14 +86,15 @@ def compare_atoms(atoms1, atoms2, tol=1e-15, excluded_properties=None):
 
         properties_to_check = set(all_changes)
         if excluded_properties:
-             properties_to_check -= set(excluded_properties)
+            properties_to_check -= set(excluded_properties)
 
         # Check properties that aren't in Atoms.arrays but are attributes of
         # Atoms objects
         for prop in ['cell', 'pbc']:
             if prop in properties_to_check:
                 properties_to_check.remove(prop)
-                if not equal(getattr(atoms1, prop), getattr(atoms2, prop), tol):
+                if not equal(getattr(atoms1, prop), getattr(atoms2, prop),
+                             atol=tol):
                     system_changes.append(prop)
 
         arrays1 = set(atoms1.arrays)
@@ -103,10 +108,10 @@ def compare_atoms(atoms1, atoms2, tol=1e-15, excluded_properties=None):
         system_changes += properties_to_check & (arrays1 ^ arrays2)
 
         # Finally, check all of the non-excluded properties shared by the atoms
-        # arrays
+        # arrays.
         for prop in properties_to_check & arrays1 & arrays2:
-                if not equal(atoms1.arrays[prop], atoms2.arrays[prop], tol):
-                    system_changes.append(prop)
+            if not equal(atoms1.arrays[prop], atoms2.arrays[prop], atol=tol):
+                system_changes.append(prop)
 
     return system_changes
 
@@ -121,11 +126,11 @@ all_changes = ['positions', 'numbers', 'cell', 'pbc',
 
 # Recognized names of calculators sorted alphabetically:
 names = ['abinit', 'ace', 'aims', 'amber', 'asap', 'castep', 'cp2k',
-         'crystal', 'demon', 'demonnano', 'dftb', 'dftd3', 'dmol', 'eam', 'elk',
-         'emt', 'espresso', 'exciting', 'ff', 'fleur', 'gaussian',
-         'gpaw', 'gromacs', 'gulp', 'hotbit', 'jacapo', 'kim',
+         'crystal', 'demon', 'demonnano', 'dftb', 'dftd3', 'dmol', 'eam',
+         'elk', 'emt', 'espresso', 'exciting', 'ff', 'fleur', 'gamess_us',
+         'gaussian', 'gpaw', 'gromacs', 'gulp', 'hotbit', 'kim',
          'lammpslib', 'lammpsrun', 'lj', 'mopac', 'morse', 'nwchem',
-         'octopus', 'onetep', 'openmx', 'psi4', 'qchem', 'siesta',
+         'octopus', 'onetep', 'openmx', 'orca', 'psi4', 'qchem', 'siesta',
          'tip3p', 'tip4p', 'turbomole', 'vasp']
 
 
@@ -139,6 +144,7 @@ special = {'cp2k': 'CP2K',
            'crystal': 'CRYSTAL',
            'ff': 'ForceField',
            'fleur': 'FLEUR',
+           'gamess_us': 'GAMESSUS',
            'gulp': 'GULP',
            'kim': 'KIM',
            'lammpsrun': 'LAMMPS',
@@ -148,6 +154,7 @@ special = {'cp2k': 'CP2K',
            'morse': 'MorsePotential',
            'nwchem': 'NWChem',
            'openmx': 'OpenMX',
+           'orca': 'ORCA',
            'qchem': 'QChem',
            'tip3p': 'TIP3P',
            'tip4p': 'TIP4P'}
@@ -187,27 +194,42 @@ def get_calculator_class(name):
     return Calculator
 
 
-def equal(a, b, tol=None):
+def equal(a, b, tol=None, rtol=None, atol=None):
     """ndarray-enabled comparison function."""
     # XXX Known bugs:
     #  * Comparing cell objects (pbc not part of array representation)
     #  * Infinite recursion for cyclic dicts
     #  * Can of worms is open
-    if tol is None:
-        return np.array_equal(a, b)
+    if tol is not None:
+        msg = 'Use `equal(a, b, rtol=..., atol=...)` instead of `tol=...`'
+        warnings.warn(msg, DeprecationWarning)
+        assert rtol is None and atol is None, \
+            'Do not use deprecated `tol` with `atol` and/or `rtol`'
+        rtol = tol
+        atol = tol
 
-    shape = np.shape(a)
-    if shape != np.shape(b):
+    a_is_dict = isinstance(a, dict)
+    b_is_dict = isinstance(b, dict)
+    if a_is_dict or b_is_dict:
+        # Check that both a and b are dicts
+        if not (a_is_dict and b_is_dict):
+            return False
+        if a.keys() != b.keys():
+            return False
+        return all(equal(a[key], b[key], rtol=rtol, atol=atol) for key in a)
+
+    if np.shape(a) != np.shape(b):
         return False
 
-    if not shape:
-        if isinstance(a, dict) and isinstance(b, dict):
-            if a.keys() != b.keys():
-                return False
-            return all(equal(a[key], b[key], tol) for key in a.keys())
-        return abs(a - b) < tol * abs(b) + tol
+    if rtol is None and atol is None:
+        return np.array_equal(a, b)
 
-    return np.allclose(a, b, rtol=tol, atol=tol)
+    if rtol is None:
+        rtol = 0
+    if atol is None:
+        atol = 0
+
+    return np.allclose(a, b, rtol=rtol, atol=atol)
 
 
 def kptdensity2monkhorstpack(atoms, kptdensity=3.5, even=True):
@@ -301,6 +323,7 @@ def kpts2sizeandoffsets(size=None, density=None, gamma=None, even=None,
 
     return size, offsets
 
+
 @jsonable('kpoints')
 class KPoints:
     def __init__(self, kpts=None):
@@ -419,7 +442,7 @@ class Parameters(dict):
         file.close()
 
 
-class Calculator(object):
+class Calculator:
     """Base-class for all ASE calculators.
 
     A calculator must raise PropertyNotImplementedError if asked for a
@@ -432,23 +455,31 @@ class Calculator(object):
     'magmom' and 'magmoms'.
     """
 
-    implemented_properties = []
+    implemented_properties: List[str] = []
     'Properties calculator can handle (energy, forces, ...)'
 
-    default_parameters = {}
+    default_parameters: Dict[str, Any] = {}
     'Default parameters'
 
+    ignored_changes: Set[str] = set()
+    'Properties of Atoms which we ignore for the purposes of cache '
+    'invalidation with check_state().'
+
+    discard_results_on_any_change = False
+    'Whether we purge the results following any change in the set() method.  '
+    'Most (file I/O) calculators will probably want this.'
+
     def __init__(self, restart=None, ignore_bad_restart_file=False, label=None,
                  atoms=None, directory='.', **kwargs):
         """Basic calculator implementation.
 
         restart: str
-            Prefix for restart file.  May contain a directory.  Default
+            Prefix for restart file.  May contain a directory. Default
             is None: don't restart.
         ignore_bad_restart_file: bool
             Ignore broken or missing restart file.  By default, it is an
             error if the restart file is missing or broken.
-        directory: str
+        directory: str or PurePath
             Working directory in which to read and write files and
             perform calculations.
         label: str
@@ -463,6 +494,7 @@ class Calculator(object):
         self.atoms = None  # copy of atoms object from last calculation
         self.results = {}  # calculated properties (energy, forces, ...)
         self.parameters = None  # calculational parameters
+        self._directory = None  # Initialize
 
         if restart is not None:
             try:
@@ -476,12 +508,12 @@ class Calculator(object):
         self.directory = directory
         self.prefix = None
         if label is not None:
-            if directory != '.' and '/' in label:
+            if self.directory != '.' and '/' in label:
                 raise ValueError('Directory redundantly specified though '
                                  'directory="{}" and label="{}".  '
                                  'Please omit "/" in label.'
-                                 .format(directory, label))
-            self.set_label(label)
+                                 .format(self.directory, label))
+            self.label = '/'.join((self.directory, label))
 
         if self.parameters is None:
             # Use default parameters if they were not read from file:
@@ -502,6 +534,14 @@ class Calculator(object):
         if not hasattr(self, 'name'):
             self.name = self.__class__.__name__.lower()
 
+    @property
+    def directory(self) -> str:
+        return self._directory
+
+    @directory.setter
+    def directory(self, directory: Union[str, pathlib.PurePath]):
+        self._directory = str(pathlib.Path(directory))  # Normalize path.
+
     @property
     def label(self):
         if self.directory == '.':
@@ -544,10 +584,7 @@ class Calculator(object):
         * label='abc': (directory='.', prefix='abc')
         * label='dir1/abc': (directory='dir1', prefix='abc')
         * label=None: (directory='.', prefix=None)
-
-        Calculators that must write results to files with fixed names
-        can override this method so that the directory is set to all
-        of label."""
+        """
         self.label = label
 
     def get_default_parameters(self):
@@ -632,11 +669,14 @@ class Calculator(object):
                 changed_parameters[key] = value
                 self.parameters[key] = value
 
+        if self.discard_results_on_any_change and changed_parameters:
+            self.reset()
         return changed_parameters
 
     def check_state(self, atoms, tol=1e-15):
         """Check for any system changes since last calculation."""
-        return compare_atoms(self.atoms, atoms, tol)
+        return compare_atoms(self.atoms, atoms, tol=tol,
+                             excluded_properties=set(self.ignored_changes))
 
     def get_potential_energy(self, atoms=None, force_consistent=False):
         energy = self.get_property('energy', atoms)
@@ -662,6 +702,9 @@ class Calculator(object):
         return self.get_property('stress', atoms)
 
     def get_stresses(self, atoms=None):
+        """the calculator should return intensive stresses, i.e., such that
+                stresses.sum(axis=0) == stress
+        """
         return self.get_property('stresses', atoms)
 
     def get_dipole_moment(self, atoms=None):
@@ -748,11 +791,14 @@ class Calculator(object):
                             'magmoms': np.zeros(len(atoms))}
 
         The subclass implementation should first call this
-        implementation to set the atoms attribute.
+        implementation to set the atoms attribute and create any missing
+        directories.
         """
 
         if atoms is not None:
             self.atoms = atoms.copy()
+        if not os.path.isdir(self._directory):
+            os.makedirs(self._directory)
 
     def calculate_numerical_forces(self, atoms, d=0.001):
         """Calculate numerical forces using finite difference.
@@ -808,7 +854,7 @@ class Calculator(object):
 
     def band_structure(self):
         """Create band-structure object for plotting."""
-        from ase.dft.band_structure import get_band_structure
+        from ase.spectrum.band_structure import get_band_structure
         # XXX This calculator is supposed to just have done a band structure
         # calculation, but the calculator may not have the correct Fermi level
         # if it updated the Fermi level after changing k-points.
@@ -817,11 +863,30 @@ class Calculator(object):
         # from the selfconsistent calculation.
         return get_band_structure(calc=self)
 
+    def calculate_properties(self, atoms, properties):
+        """This method is experimental; currently for internal use."""
+        for name in properties:
+            if name not in all_outputs:
+                raise ValueError(f'No such property: {name}')
+
+        # We ignore system changes for now.
+        self.calculate(atoms, properties, system_changes=all_changes)
+
+        props = self.export_properties()
+
+        for name in properties:
+            if name not in props:
+                raise PropertyNotPresent(name)
+        return props
+
+    def export_properties(self):
+        return Properties(self.results)
+
 
 class FileIOCalculator(Calculator):
     """Base class for calculators that write/read input/output files."""
 
-    command = None  # str
+    command: Optional[str] = None
     'Command used to start calculation'
 
     def __init__(self, restart=None, ignore_bad_restart_file=False,
