@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """This module defines an ASE interface to CP2K.
 
 https://www.cp2k.org/
@@ -14,7 +12,8 @@ from subprocess import Popen, PIPE
 import numpy as np
 import ase.io
 from ase.units import Rydberg
-from ase.calculators.calculator import Calculator, all_changes, Parameters
+from ase.calculators.calculator import (Calculator, all_changes, Parameters,
+                                        CalculatorSetupError)
 
 
 class CP2K(Calculator):
@@ -49,7 +48,6 @@ class CP2K(Calculator):
 
     >>> CP2K.command="env OMP_NUM_THREADS=2 mpiexec -np 4 cp2k_shell.psmp"
 
-
     Arguments:
 
     auto_write: bool
@@ -91,6 +89,28 @@ class CP2K(Calculator):
         gives access to all features of CP2K.
         Note, that most keywords accept ``None`` to disable the generation
         of the corresponding input section.
+
+        This input template is important for advanced CP2K
+        inputs, but is also needed for e.g. controlling the Brillouin
+        zone integration. The example below illustrates some common
+        options::
+
+           >>> inp = '''&FORCE_EVAL
+           >>>    &DFT
+           >>>      &KPOINTS
+           >>>        SCHEME MONKHORST-PACK 12 12 8
+           >>>      &END KPOINTS
+           >>>      &SCF
+           >>>        ADDED_MOS 10
+           >>>        &SMEAR
+           >>>          METHOD FERMI_DIRAC
+           >>>          ELECTRONIC_TEMPERATURE [K] 500.0
+           >>>        &END SMEAR
+           >>>      &END SCF
+           >>>    &END DFT
+           >>>  &END FORCE_EVAL
+           >>>  '''
+
     max_scf: int
         Maximum number of SCF iteration to be performed for
         one optimization. Default is ``50``.
@@ -121,13 +141,12 @@ class CP2K(Calculator):
     print_level: str
         PRINT_LEVEL of global output.
         Possible options are:
-        DEBUG Everything is written out, useful for debugging purposes only 
-        HIGH Lots of output 
-        LOW Little output 
-        MEDIUM Quite some output 
-        SILENT Almost no output 
+        DEBUG Everything is written out, useful for debugging purposes only
+        HIGH Lots of output
+        LOW Little output
+        MEDIUM Quite some output
+        SILENT Almost no output
         Default is 'LOW'
-        
     """
 
     implemented_properties = ['energy', 'free_energy', 'forces', 'stress']
@@ -196,6 +215,13 @@ class CP2K(Calculator):
 
     def set(self, **kwargs):
         """Set parameters like set(key1=value1, key2=value2, ...)."""
+        msg = '"%s" is not a known keyword for the CP2K calculator. ' \
+              'To access all features of CP2K by means of an input ' \
+              'template, consider using the "inp" keyword instead.'
+        for key in kwargs:
+            if key not in self.default_parameters:
+                raise CalculatorSetupError(msg % key)
+
         changed_parameters = Calculator.set(self, **kwargs)
         if changed_parameters:
             self.reset()
@@ -203,7 +229,7 @@ class CP2K(Calculator):
     def write(self, label):
         'Write atoms, parameters and calculated results into restart files.'
         if self._debug:
-            print("Writting restart to: ", label)
+            print("Writing restart to: ", label)
         self.atoms.write(label + '_restart.traj')
         self.parameters.write(label + '_params.ase')
         from ase.io.jsonio import write_json
@@ -439,7 +465,7 @@ class CP2K(Calculator):
         return '\n'.join(output_lines)
 
 
-class Cp2kShell(object):
+class Cp2kShell:
     """Wrapper for CP2K-shell child-process"""
 
     def __init__(self, command, debug):
@@ -514,7 +540,7 @@ class Cp2kShell(object):
         received = self.recv()
         assert received == line
 
-class InputSection(object):
+class InputSection:
     """Represents a section of a CP2K input file"""
     def __init__(self, name, params=None):
         self.name = name.upper()
