@@ -23,6 +23,7 @@ class MOPAC(FileIOCalculator):
     default_parameters = dict(
         method='PM7',
         task='1SCF GRADIENTS',
+        charge=0,
         relscf=0.0001)
 
     methods = ['AM1', 'MNDO', 'MNDOD', 'PM3', 'PM6', 'PM6-D3', 'PM6-DH+',
@@ -83,7 +84,11 @@ class MOPAC(FileIOCalculator):
             s += 'RELSCF={0} '.format(p.relscf)
 
         # Write charge:
-        charge = atoms.get_initial_charges().sum()
+        if p.charge:
+            charge = p.charge
+        else:
+            charge = atoms.get_initial_charges().sum()
+            
         if charge != 0:
             s += 'CHARGE={0} '.format(int(round(charge)))
 
