@@ -32,18 +32,32 @@ from ase.calculators.siesta.parameters import format_fdf
 meV = 0.001 * eV
 
 
-def get_siesta_version(command):
+def parse_siesta_version(output: bytes) -> str:
+    match = re.search(rb'Siesta Version\s*:\s*(\S+)', output)
+
+    if match is None:
+        raise RuntimeError('Could not get Siesta version info from output '
+                           '{!r}'.format(output))
+
+    string = match.group(1).decode('ascii')
+    return string
+
+
+def get_siesta_version(executable: str) -> str:
     """ Return SIESTA version number.
 
     Run the command, for instance 'siesta' and
     then parse the output in order find the
     version number.
     """
+    # XXX We need a test of this kind of function.  But Siesta().command
+    # is not enough to tell us how to run Siesta, because it could contain
+    # all sorts of mpirun and other weird parts.
 
     temp_dirname = tempfile.mkdtemp(prefix='siesta-version-check-')
     try:
         from subprocess import Popen, PIPE
-        proc = Popen(command,
+        proc = Popen([executable],
                      stdin=PIPE,
                      stdout=PIPE,
                      stderr=PIPE,
@@ -55,14 +69,7 @@ def get_siesta_version(command):
     finally:
         shutil.rmtree(temp_dirname)
 
-    match = re.search(rb'Siesta Version\s*:\s*(\S+)', output)
-
-    if match is None:
-        raise RuntimeError('Could not get Siesta version info from output '
-                           '{!r}'.format(output))
-
-    string = match.group(1).decode('ascii')
-    return string
+    return parse_siesta_version(output)
 
 
 def bandpath2bandpoints(path):
@@ -122,7 +129,7 @@ def resolve_band_structure(path, kpts, energies, efermi):
     #
     # Also we should perhaps verify the cell.  If we had the cell, we
     # could construct the bandpath from scratch (i.e., pure outputs).
-    from ase.dft.band_structure import BandStructure
+    from ase.spectrum.band_structure import BandStructure
     ksn2e = energies
     skn2e = np.swapaxes(ksn2e, 0, 1)
     bs = BandStructure(path, skn2e, reference=efermi)
@@ -161,7 +168,14 @@ class SiestaParameters(Parameters):
 class Siesta(FileIOCalculator):
     """Calculator interface to the SIESTA code.
     """
-    allowed_basis_names = ['SZ', 'SZP', 'DZ', 'DZP', 'TZP']
+    # Siesta manual does not document many of the basis names.
+    # basis_specs.f has a ton of aliases for each.
+    # Let's just list one of each type then.
+    #
+    # Maybe we should be less picky about these keyword names.
+    allowed_basis_names = ['SZ', 'SZP',
+                           'DZ', 'DZP', 'DZP2',
+                           'TZ', 'TZP', 'TZP2', 'TZP3']
     allowed_spins = ['non-polarized', 'collinear',
                      'non-collinear', 'spin-orbit']
     allowed_xc = {
@@ -173,20 +187,20 @@ class Siesta(FileIOCalculator):
 
     name = 'siesta'
     command = 'siesta < PREFIX.fdf > PREFIX.out'
-    implemented_properties = (
+    implemented_properties = [
         'energy',
         'forces',
         'stress',
         'dipole',
         'eigenvalues',
         'density',
-        'fermi_energy')
+        'fermi_energy']
 
     # Dictionary of valid input vaiables.
     default_parameters = SiestaParameters()
 
     # XXX Not a ASE standard mechanism (yet).  We need to communicate to
-    # ase.dft.band_structure.calculate_band_structure() that we expect
+    # ase.spectrum.band_structure.calculate_band_structure() that we expect
     # it to use the bandpath keyword.
     accepts_bandpath_keyword = True
 
@@ -346,6 +360,12 @@ class Siesta(FileIOCalculator):
                            SiestaParameters.
         """
 
+        # XXX Inserted these next few lines because set() would otherwise
+        # discard all previously set keywords to their defaults!  --askhl
+        current = self.parameters.copy()
+        current.update(kwargs)
+        kwargs = current
+
         # Find not allowed keys.
         default_keys = list(self.__class__.default_parameters)
         offending_keys = set(kwargs) - set(default_keys)
@@ -796,7 +816,7 @@ class Siesta(FileIOCalculator):
         """
         species, species_numbers = self.species(atoms)
 
-        if not self['pseudo_path'] is None:
+        if self['pseudo_path'] is not None:
             pseudo_path = self['pseudo_path']
         elif 'SIESTA_PP_PATH' in os.environ:
             pseudo_path = os.environ['SIESTA_PP_PATH']
@@ -1284,7 +1304,7 @@ class Siesta(FileIOCalculator):
                         'DM.NumberPulay': 4,
                         'XML.Write': True})
 
-        Na8.set_calculator(siesta)
+        Na8.calc = siesta
         e = Na8.get_potential_energy()
         freq, pol = siesta.get_polarizability_pyscf_inter(
             label="siesta",
@@ -1379,7 +1399,7 @@ class Siesta(FileIOCalculator):
             **kw):
         r"""
         Perform TDDFT calculation using the pyscf.nao module for a molecule.
-        The external pertubation is created by a electron moving at
+        The external perturbation is created by a electron moving at
         the velocity velec and with an impact parameter b.
 
         Parameters
@@ -1464,7 +1484,7 @@ class Siesta(FileIOCalculator):
                 'XML.Write': True})
 
 
-        Na8.set_calculator(siesta)
+        Na8.calc = siesta
         e = Na8.get_potential_energy()
         tddft = siesta.pyscf_tddft_eels(
             label="siesta", jcutoff=7, iter_broadening=0.15/Ha,
@@ -1537,7 +1557,7 @@ class Siesta(FileIOCalculator):
         Parameters
         ----------
         mbpt_inp : dict, optional
-            dictionnary of the input for the mbpt_lcao program
+            dictionary of the input for the mbpt_lcao program
             (http://mbpt-domiprod.wikidot.com/list-of-parameters)
             if mbpt_inp is None, the function read the output file
             from a previous mbpt_lcao run.
@@ -1640,7 +1660,7 @@ class Siesta(FileIOCalculator):
                     'gwa_initialization':'SIESTA_PB'}
 
 
-        Na8.set_calculator(siesta)
+        Na8.calc = siesta
         e = Na8.get_potential_energy() #run siesta
         freq, pol = siesta.get_polarizability_siesta(mbpt_inp,
                                                      format_output='txt',
