@@ -1,3 +1,4 @@
+# type: ignore
 """
 This module defines an ASE interface to Turbomole: http://www.turbomole.com/
 
@@ -18,7 +19,6 @@ from ase.units import Ha, Bohr
 from ase.io import read, write
 from ase.calculators.calculator import FileIOCalculator
 from ase.calculators.calculator import PropertyNotImplementedError, ReadError
-from ase.utils import basestring
 
 
 def read_output(regex):
@@ -42,7 +42,7 @@ def execute(args, input_str=None, error_test=True,
             stdout_tofile=True):
     """executes a turbomole executable and process the outputs"""
 
-    if isinstance(args, basestring):
+    if isinstance(args, str):
         args = args.split()
 
     if stdout_tofile:
@@ -616,7 +616,7 @@ class Turbomole(FileIOCalculator):
             self.reset()
 
         if atoms is not None:
-            atoms.set_calculator(self)
+            atoms.calc = self
             self.set_atoms(atoms)
 
     def __getitem__(self, item):
@@ -762,7 +762,7 @@ class Turbomole(FileIOCalculator):
 
         # kwargs parameters are ignored if user provides define_str
         if self.define_str is not None:
-            assert isinstance(self.define_str, basestring)
+            assert isinstance(self.define_str, str)
             assert len(self.define_str) != 0
             return
 
@@ -1105,7 +1105,7 @@ class Turbomole(FileIOCalculator):
     def read_restart(self):
         """read a previous calculation from control file"""
         self.atoms = read('coord')
-        self.atoms.set_calculator(self)
+        self.atoms.calc = self
         self.converged = self.read_convergence()
         read_methods = [
             self.read_energy,
@@ -1188,7 +1188,7 @@ class Turbomole(FileIOCalculator):
                 else:
                     if pdgs[p] is None:
                         params[p] = None
-                    elif isinstance(pdgs[p], basestring):
+                    elif isinstance(pdgs[p], str):
                         if self.parameter_type[p] is bool:
                             params[p] = (pdgs[p] == self.parameter_key[p])
                     else:
