@@ -1,4 +1,3 @@
-
 import json
 import numpy as np
 
@@ -10,7 +9,7 @@ from ase.dft.kpoints import (get_monkhorst_pack_size_and_offset,
                              monkhorst_pack_interpolate,
                              bandpath, BandPath,
                              get_special_points)
-from ase.dft.band_structure import BandStructure
+from ase.spectrum.band_structure import BandStructure
 
 
 class CLICommand:
@@ -35,6 +34,7 @@ class CLICommand:
         parser.add_argument('-n', '--points', type=int, default=100,
                             help='Number of points along the path '
                             '(default: 100)')
+        parser.add_argument('-o', '--output', help='Write image to a file')
         parser.add_argument('-r', '--range', nargs=2, default=['-3', '3'],
                             metavar=('emin', 'emax'),
                             help='Default: "-3.0 3.0" '
@@ -44,6 +44,7 @@ class CLICommand:
     def run(args, parser):
         main(args, parser)
 
+
 def atoms2bandstructure(atoms, parser, args):
     cell = atoms.get_cell()
     calc = atoms.calc
@@ -87,13 +88,14 @@ def atoms2bandstructure(atoms, parser, args):
                             '\nMaybe you want its special path:'
                             ' {}'.format(cs, kptpath))
                 parser.error(err)
-        bz2ibz = calc.get_bz_to_ibz_map()
+            bz2ibz = calc.get_bz_to_ibz_map()
 
-        path_kpts = bandpath(args.path, atoms.cell, args.points)[0]
-        icell = atoms.get_reciprocal_cell()
-        eps = monkhorst_pack_interpolate(path_kpts, eps.transpose(1, 0, 2),
-                                         icell, bz2ibz, size, offset)
-        eps = eps.transpose(1, 0, 2)
+            path_kpts = bandpath(args.path, atoms.cell, args.points).kpts
+
+            icell = atoms.get_reciprocal_cell()
+            eps = monkhorst_pack_interpolate(path_kpts, eps.transpose(1, 0, 2),
+                                             icell, bz2ibz, size, offset)
+            eps = eps.transpose(1, 0, 2)
 
     special_points = get_special_points(cell)
     path = BandPath(atoms.cell, kpts=path_kpts,
@@ -123,6 +125,7 @@ def read_band_structure(args, parser):
 
     return bs
 
+
 def main(args, parser):
     import matplotlib.pyplot as plt
     bs = read_band_structure(args, parser)
@@ -130,5 +133,9 @@ def main(args, parser):
     fig = plt.gcf()
     fig.canvas.set_window_title(args.calculation)
     ax = fig.gca()
-    bs.plot(ax=ax, emin=emin, emax=emax)
-    plt.show()
+    bs.plot(ax=ax,
+            filename=args.output,
+            emin=emin + bs.reference,
+            emax=emax + bs.reference)
+    if args.output is None:
+        plt.show()
