@@ -24,8 +24,8 @@ def match(word, *suffixes):
 # Beginning of computer generated data:
 commands = {
     'band-structure':
-        ['-q', '--quiet', '-k', '--path', '-n', '--points', '-r',
-         '--range'],
+        ['-q', '--quiet', '-k', '--path', '-n', '--points', '-o',
+         '--output', '-r', '--range'],
     'build':
         ['-M', '--magnetic-moment', '--modify', '-V', '--vacuum', '-v',
          '--vacuum0', '--unit-cell', '--bond-length', '-x',
@@ -39,19 +39,25 @@ commands = {
          '--output-format', '-f', '--force', '-n',
          '--image-number', '-e', '--exec-code', '-E',
          '--exec-file', '-a', '--arrays', '-I', '--info', '-s',
-         '--split-output'],
+         '--split-output', '--read-args', '--write-args'],
     'db':
         ['-v', '--verbose', '-q', '--quiet', '-n', '--count', '-l',
          '--long', '-i', '--insert-into', '-a',
          '--add-from-file', '-k', '--add-key-value-pairs', '-L',
          '--limit', '--offset', '--delete', '--delete-keys',
          '-y', '--yes', '--explain', '-c', '--columns', '-s',
-         '--sort', '--cut', '-p', '--plot', '-P', '--plot-data',
-         '--csv', '-w', '--open-web-browser', '--no-lock-file',
-         '--analyse', '-j', '--json', '-m', '--show-metadata',
+         '--sort', '--cut', '-p', '--plot', '--csv', '-w',
+         '--open-web-browser', '--no-lock-file', '--analyse',
+         '-j', '--json', '-m', '--show-metadata',
          '--set-metadata', '-M', '--metadata-from-python-script',
          '--unique', '--strip-data', '--show-keys',
-         '--show-values', '--write-summary-files'],
+         '--show-values'],
+    'diff':
+        ['-r', '--rank-order', '-c', '--calculator-outputs',
+         '--max-lines', '-t', '--template', '--template-help',
+         '-s', '--summary-functions', '--log-file', '--as-csv'],
+    'dimensionality':
+        ['--display-all', '--no-merge'],
     'eos':
         ['-p', '--plot', '-t', '--type'],
     'find':
@@ -63,6 +69,8 @@ commands = {
          '--interpolate', '-b', '--bonds', '-s', '--scale'],
     'info':
         ['-v', '--verbose', '--formats', '--calculators'],
+    'nebplot':
+        ['--nimages', '--share-x', '--share-y'],
     'nomad-get':
         [],
     'nomad-upload':
@@ -77,8 +85,10 @@ commands = {
          '--maximum-stress', '-E', '--equation-of-state',
          '--eos-type', '-o', '--output', '--modify', '--after'],
     'test':
-        ['-c', '--calculators', '--list', '--list-calculators', '-j',
-         '--jobs', '-v', '--verbose', '--strict'],
+        ['-c', '--calculators', '--help-calculators', '--list',
+         '--list-calculators', '-j', '--jobs', '-v', '--verbose',
+         '--strict', '--fast', '--coverage', '--nogui',
+         '--pytest'],
     'ulm':
         ['-n', '--index', '-d', '--delete', '-v', '--verbose']}
 # End of computer generated data
@@ -117,11 +127,13 @@ def complete(word, previous, line, point):
         if previous in ['-c', '--calculators']:
             from ase.calculators.calculator import names as words
         elif not word.startswith('-'):
-            # Suggest names of tests.  We suggest all matching tests.
-            # It might be better to autocomplete only up to directory
-            # names.
-            from ase.test.testsuite import get_tests
-            words = get_tests()
+            from ase.test.testsuite import all_test_modules_and_groups
+            words = []
+            for path in all_test_modules_and_groups():
+                path = str(path)
+                if not path.endswith('.py'):
+                    path += '/'
+                words.append(path)
 
     return words
 
@@ -133,10 +145,15 @@ if sys.version_info[0] == 2:
                   'Please consider rerunning \'ase completion\'.')
 
 
-word, previous = sys.argv[2:]
-line = os.environ['COMP_LINE']
-point = int(os.environ['COMP_POINT'])
-words = complete(word, previous, line, point)
-for w in words:
-    if w.startswith(word):
-        print(w)
+def main():
+    word, previous = sys.argv[2:]
+    line = os.environ['COMP_LINE']
+    point = int(os.environ['COMP_POINT'])
+    words = complete(word, previous, line, point)
+    for w in words:
+        if w.startswith(word):
+            print(w)
+
+
+if __name__ == '__main__':
+    main()
