@@ -1,9 +1,7 @@
 import platform
-import os
 import sys
 
-from ase.utils import import_module
-from ase.utils import search_current_git_hash
+from ase.dependencies import all_dependencies
 from ase.io.formats import filetype, ioformats, UnknownFileTypeError
 from ase.io.ulm import print_ulm_info
 from ase.io.bundletrajectory import print_bundletrajectory_info
@@ -77,24 +75,9 @@ class CLICommand:
 def print_info():
     versions = [('platform', platform.platform()),
                 ('python-' + sys.version.split()[0], sys.executable)]
-    for name in ['ase', 'numpy', 'scipy', 'ase_ext']:
-        try:
-            module = import_module(name)
-        except ImportError:
-            if name != 'ase_ext':
-                versions.append((name, 'no'))
-        else:
-            # Search for git hash
-            githash = search_current_git_hash(module)
-            if githash is None:
-                githash = ''
-            else:
-                githash = '-{:.10}'.format(githash)
-            versions.append((name + '-' + module.__version__ + githash,
-                            module.__file__.rsplit(os.sep, 1)[0] + os.sep))
 
-    for a, b in versions:
-        print('{:25}{}'.format(a, b))
+    for name, path in versions + all_dependencies():
+        print('{:24} {}'.format(name, path))
 
 
 def print_formats():
