@@ -1,10 +1,9 @@
-
 import argparse
 import sys
 import textwrap
+from importlib import import_module
 
 from ase import __version__
-from ase.utils import import_module
 
 
 class CLIError(Exception):
@@ -25,14 +24,18 @@ commands = [
     ('run', 'ase.cli.run'),
     ('band-structure', 'ase.cli.band_structure'),
     ('build', 'ase.cli.build'),
+    ('dimensionality', 'ase.cli.dimensionality'),
     ('eos', 'ase.eos'),
     ('ulm', 'ase.io.ulm'),
     ('find', 'ase.cli.find'),
+    ('nebplot', 'ase.cli.nebplot'),
     ('nomad-upload', 'ase.cli.nomad'),
     ('nomad-get', 'ase.cli.nomadget'),
     ('convert', 'ase.cli.convert'),
     ('reciprocal', 'ase.cli.reciprocal'),
-    ('completion', 'ase.cli.completion')]
+    ('completion', 'ase.cli.completion'),
+    ('diff', 'ase.cli.diff')
+]
 
 
 def main(prog='ase', description='ASE command line tool.',
@@ -115,6 +118,7 @@ def main(prog='ase', description='ASE command line tool.',
 
 class Formatter(argparse.HelpFormatter):
     """Improved help formatter."""
+
     def _fill_text(self, text, width, indent):
         assert indent == ''
         out = ''
@@ -136,7 +140,6 @@ class Formatter(argparse.HelpFormatter):
             out += '\n'
         return out[:-1]
 
-
 def old():
     cmd = sys.argv[0].split('-')[-1]
     print('Please use "ase {cmd}" instead of "ase-{cmd}"'.format(cmd=cmd))
