@@ -21,9 +21,11 @@ import sqlite3
 import sys
 
 import numpy as np
+from contextlib import contextmanager
 
 import ase.io.jsonio
 from ase.data import atomic_numbers
+from ase.calculators.calculator import all_properties
 from ase.db.row import AtomsRow
 from ase.db.core import (Database, ops, now, lock, invop, parse_selection,
                          object_to_bytes, bytes_to_object)
@@ -116,7 +118,7 @@ def float_if_not_none(x):
         return float(x)
 
 
-class SQLite3Database(Database, object):
+class SQLite3Database(Database):
     type = 'db'
     initialized = False
     _allow_reading_old_format = False
@@ -168,10 +170,11 @@ class SQLite3Database(Database, object):
         return array
 
     def _connect(self):
-        return sqlite3.connect(self.filename, timeout=600)
+        return sqlite3.connect(self.filename, timeout=20)
 
     def __enter__(self):
         assert self.connection is None
+        self.change_count = 0
         self.connection = self._connect()
         return self
 
@@ -183,6 +186,25 @@ class SQLite3Database(Database, object):
         self.connection.close()
         self.connection = None
 
+    @contextmanager
+    def managed_connection(self, commit_frequency=5000):
+        try:
+            con = self.connection or self._connect()
+            self._initialize(con)
+            yield con
+        except ValueError as exc:
+            if self.connection is None:
+                con.close()
+            raise exc
+        else:
+            if self.connection is None:
+                con.commit()
+                con.close()
+            else:
+                self.change_count += 1
+                if self.change_count % commit_frequency == 0:
+                    con.commit()
+
     def _initialize(self, con):
         if self.initialized:
             return
@@ -234,39 +256,31 @@ class SQLite3Database(Database, object):
     def _write(self, atoms, key_value_pairs, data, id):
         ext_tables = key_value_pairs.pop("external_tables", {})
         Database._write(self, atoms, key_value_pairs, data)
-        encode = self.encode
-
-        con = self.connection or self._connect()
-        self._initialize(con)
-        cur = con.cursor()
 
         mtime = now()
 
+        encode = self.encode
         blob = self.blob
 
-        text_key_values = []
-        number_key_values = []
-
         if not isinstance(atoms, AtomsRow):
             row = AtomsRow(atoms)
             row.ctime = mtime
             row.user = os.getenv('USER')
         else:
             row = atoms
-
             # Extract the external tables from AtomsRow
-            names = self._get_external_table_names(db_con=con)
+            names = self._get_external_table_names()
             for name in names:
                 new_table = row.get(name, {})
                 if new_table:
                     ext_tables[name] = new_table
 
-        if id:
-            self._delete(cur, [id], ['keys', 'text_key_values',
-                                     'number_key_values', 'species'])
-        else:
-            if not key_value_pairs:
-                key_value_pairs = row.key_value_pairs
+        if not id and not key_value_pairs and not ext_tables:
+            key_value_pairs = row.key_value_pairs
+
+        for k, v in ext_tables.items():
+            dtype = self._guess_type(v)
+            self._create_table_if_not_exists(k, dtype)
 
         constraints = row._constraints
         if constraints:
@@ -297,148 +311,118 @@ class SQLite3Database(Database, object):
 
         if not data:
             data = row._data
-        if not isinstance(data, (str, bytes)):
-            data = encode(data, binary=self.version >= 9)
 
-        values += (row.get('energy'),
-                   row.get('free_energy'),
-                   blob(row.get('forces')),
-                   blob(row.get('stress')),
-                   blob(row.get('dipole')),
-                   blob(row.get('magmoms')),
-                   row.get('magmom'),
-                   blob(row.get('charges')),
-                   encode(key_value_pairs),
-                   data,
-                   len(row.numbers),
-                   float_if_not_none(row.get('fmax')),
-                   float_if_not_none(row.get('smax')),
-                   float_if_not_none(row.get('volume')),
-                   float(row.mass),
-                   float(row.charge))
+        with self.managed_connection() as con:
+            if not isinstance(data, (str, bytes)):
+                data = encode(data, binary=self.version >= 9)
 
-        if id is None:
-            q = self.default + ', ' + ', '.join('?' * len(values))
-            cur.execute('INSERT INTO systems VALUES ({})'.format(q),
-                        values)
-            id = self.get_last_id(cur)
-        else:
-            q = ', '.join(name + '=?' for name in self.columnnames[1:])
-            cur.execute('UPDATE systems SET {} WHERE id=?'.format(q),
-                        values + (id,))
+            values += (row.get('energy'),
+                       row.get('free_energy'),
+                       blob(row.get('forces')),
+                       blob(row.get('stress')),
+                       blob(row.get('dipole')),
+                       blob(row.get('magmoms')),
+                       row.get('magmom'),
+                       blob(row.get('charges')),
+                       encode(key_value_pairs),
+                       data,
+                       len(row.numbers),
+                       float_if_not_none(row.get('fmax')),
+                       float_if_not_none(row.get('smax')),
+                       float_if_not_none(row.get('volume')),
+                       float(row.mass),
+                       float(row.charge))
 
-        count = row.count_atoms()
-        if count:
-            species = [(atomic_numbers[symbol], n, id)
-                       for symbol, n in count.items()]
-            cur.executemany('INSERT INTO species VALUES (?, ?, ?)',
-                            species)
-
-        text_key_values = []
-        number_key_values = []
-        for key, value in key_value_pairs.items():
-            if isinstance(value, (numbers.Real, np.bool_)):
-                number_key_values.append([key, float(value), id])
+            cur = con.cursor()
+            if id is None:
+                q = self.default + ', ' + ', '.join('?' * len(values))
+                cur.execute('INSERT INTO systems VALUES ({})'.format(q),
+                            values)
+                id = self.get_last_id(cur)
             else:
-                assert isinstance(value, str)
-                text_key_values.append([key, value, id])
+                self._delete(cur, [id], ['keys', 'text_key_values',
+                                         'number_key_values', 'species'])
+                q = ', '.join(name + '=?' for name in self.columnnames[1:])
+                cur.execute('UPDATE systems SET {} WHERE id=?'.format(q),
+                            values + (id,))
 
-        cur.executemany('INSERT INTO text_key_values VALUES (?, ?, ?)',
-                        text_key_values)
-        cur.executemany('INSERT INTO number_key_values VALUES (?, ?, ?)',
-                        number_key_values)
-        cur.executemany('INSERT INTO keys VALUES (?, ?)',
-                        [(key, id) for key in key_value_pairs])
+            count = row.count_atoms()
+            if count:
+                species = [(atomic_numbers[symbol], n, id)
+                           for symbol, n in count.items()]
+                cur.executemany('INSERT INTO species VALUES (?, ?, ?)',
+                                species)
 
-        # Update external tables
-        valid_entries = []
-        for k, v in ext_tables.items():
-            try:
-                # Guess the type of the value
-                dtype = self._guess_type(v)
-                self._create_table_if_not_exists(k, dtype, db_con=con)
-                v["id"] = id
-                valid_entries.append(k)
-            except ValueError as exc:
-                # Close the connection without committing
-                if self.connection is None:
-                    con.close()
-                # Raise error again
-                raise ValueError(exc)
+            text_key_values = []
+            number_key_values = []
+            for key, value in key_value_pairs.items():
+                if isinstance(value, (numbers.Real, np.bool_)):
+                    number_key_values.append([key, float(value), id])
+                else:
+                    assert isinstance(value, str)
+                    text_key_values.append([key, value, id])
 
-        # Insert entries in the valid tables
-        for tabname in valid_entries:
-            try:
+            cur.executemany('INSERT INTO text_key_values VALUES (?, ?, ?)',
+                            text_key_values)
+            cur.executemany('INSERT INTO number_key_values VALUES (?, ?, ?)',
+                            number_key_values)
+            cur.executemany('INSERT INTO keys VALUES (?, ?)',
+                            [(key, id) for key in key_value_pairs])
+
+            # Insert entries in the valid tables
+            for tabname in ext_tables.keys():
+                entries = ext_tables[tabname]
+                entries['id'] = id
                 self._insert_in_external_table(
                     cur, name=tabname, entries=ext_tables[tabname])
-            except ValueError as exc:
-                # Close the connection without committing
-                if self.connection is None:
-                    con.close()
-                # Raise the error again
-                raise ValueError(exc)
-
-        if self.connection is None:
-            con.commit()
-            con.close()
 
         return id
 
     def _update(self, id, key_value_pairs, data=None):
         """Update key_value_pairs and data for a single row """
         encode = self.encode
-        ext_tab = key_value_pairs.pop('external_tables', {})
+        ext_tables = key_value_pairs.pop('external_tables', {})
 
-        con = self.connection or self._connect()
-        self._initialize(con)
-        cur = con.cursor()
+        for k, v in ext_tables.items():
+            dtype = self._guess_type(v)
+            self._create_table_if_not_exists(k, dtype)
 
         mtime = now()
+        with self.managed_connection() as con:
+            cur = con.cursor()
+            cur.execute(
+                'UPDATE systems SET mtime=?, key_value_pairs=? WHERE id=?',
+                (mtime, encode(key_value_pairs), id))
+            if data:
+                if not isinstance(data, (str, bytes)):
+                    data = encode(data, binary=self.version >= 9)
+                cur.execute('UPDATE systems set data=? where id=?', (data, id))
 
-        cur.execute(
-            'UPDATE systems SET mtime=?, key_value_pairs=? WHERE id=?',
-            (mtime, encode(key_value_pairs), id))
-        if data:
-            if not isinstance(data, (str, bytes)):
-                data = encode(data, binary=self.version >= 9)
-            cur.execute('UPDATE systems set data=? where id=?', (data, id))
+            self._delete(cur, [id], ['keys', 'text_key_values',
+                                     'number_key_values'])
 
-        self._delete(cur, [id], ['keys', 'text_key_values',
-                                 'number_key_values'])
+            text_key_values = []
+            number_key_values = []
+            for key, value in key_value_pairs.items():
+                if isinstance(value, (numbers.Real, np.bool_)):
+                    number_key_values.append([key, float(value), id])
+                else:
+                    assert isinstance(value, str)
+                    text_key_values.append([key, value, id])
 
-        text_key_values = []
-        number_key_values = []
-        for key, value in key_value_pairs.items():
-            if isinstance(value, (numbers.Real, np.bool_)):
-                number_key_values.append([key, float(value), id])
-            else:
-                assert isinstance(value, str)
-                text_key_values.append([key, value, id])
+            cur.executemany('INSERT INTO text_key_values VALUES (?, ?, ?)',
+                            text_key_values)
+            cur.executemany('INSERT INTO number_key_values VALUES (?, ?, ?)',
+                            number_key_values)
+            cur.executemany('INSERT INTO keys VALUES (?, ?)',
+                            [(key, id) for key in key_value_pairs])
 
-        cur.executemany('INSERT INTO text_key_values VALUES (?, ?, ?)',
-                        text_key_values)
-        cur.executemany('INSERT INTO number_key_values VALUES (?, ?, ?)',
-                        number_key_values)
-        cur.executemany('INSERT INTO keys VALUES (?, ?)',
-                        [(key, id) for key in key_value_pairs])
-
-        for tabname, values in ext_tab.items():
-            try:
-                dtype = self._guess_type(values)
-                values['id'] = id
-                self._create_table_if_not_exists(tabname, dtype, db_con=con)
+            # Insert entries in the valid tables
+            for tabname in ext_tables.keys():
+                entries = ext_tables[tabname]
+                entries['id'] = id
                 self._insert_in_external_table(
-                    cur, name=tabname, entries=values)
-            except ValueError as exc:
-                # Close the connection without committing
-                if self.connection is None:
-                    con.close()
-                # Raise the error again
-                raise ValueError(exc)
-
-        if self.connection is None:
-            con.commit()
-            con.close()
+                    cur, name=tabname, entries=ext_tables[tabname])
 
         return id
 
@@ -452,16 +436,15 @@ class SQLite3Database(Database, object):
             return 0
 
     def _get_row(self, id):
-        con = self._connect()
-        self._initialize(con)
-        c = con.cursor()
-        if id is None:
-            c.execute('SELECT COUNT(*) FROM systems')
-            assert c.fetchone()[0] == 1
-            c.execute('SELECT * FROM systems')
-        else:
-            c.execute('SELECT * FROM systems WHERE id=?', (id,))
-        values = c.fetchone()
+        with self.managed_connection() as con:
+            cur = con.cursor()
+            if id is None:
+                cur.execute('SELECT COUNT(*) FROM systems')
+                assert cur.fetchone()[0] == 1
+                cur.execute('SELECT * FROM systems')
+            else:
+                cur.execute('SELECT * FROM systems WHERE id=?', (id,))
+            values = cur.fetchone()
 
         return self._convert_tuple_to_row(values)
 
@@ -643,8 +626,6 @@ class SQLite3Database(Database, object):
     def _select(self, keys, cmps, explain=False, verbosity=0,
                 limit=None, offset=0, sort=None, include_data=True,
                 columns='all'):
-        con = self._connect()
-        self._initialize(con)
 
         values = np.array([None for i in range(27)])
         values[25] = '{}'
@@ -704,29 +685,30 @@ class SQLite3Database(Database, object):
         if verbosity == 2:
             print(sql, args)
 
-        cur = con.cursor()
-        cur.execute(sql, args)
-        if explain:
-            for row in cur.fetchall():
-                yield {'explain': row}
-        else:
-            n = 0
-            for shortvalues in cur.fetchall():
-                values[columnindex] = shortvalues
-                yield self._convert_tuple_to_row(tuple(values))
-                n += 1
+        with self.managed_connection() as con:
+            cur = con.cursor()
+            cur.execute(sql, args)
+            if explain:
+                for row in cur.fetchall():
+                    yield {'explain': row}
+            else:
+                n = 0
+                for shortvalues in cur.fetchall():
+                    values[columnindex] = shortvalues
+                    yield self._convert_tuple_to_row(tuple(values))
+                    n += 1
 
-            if sort and sort_table != 'systems':
-                # Yield rows without sort key last:
-                if limit is not None:
-                    if n == limit:
-                        return
-                    limit -= n
-                for row in self._select(keys + ['-' + sort], cmps,
-                                        limit=limit, offset=offset,
-                                        include_data=include_data,
-                                        columns=columns):
-                    yield row
+                if sort and sort_table != 'systems':
+                    # Yield rows without sort key last:
+                    if limit is not None:
+                        if n == limit:
+                            return
+                        limit -= n
+                    for row in self._select(keys + ['-' + sort], cmps,
+                                            limit=limit, offset=offset,
+                                            include_data=include_data,
+                                            columns=columns):
+                        yield row
 
     def get_offset_string(self, offset, limit=None):
         sql = ''
@@ -741,28 +723,26 @@ class SQLite3Database(Database, object):
     def count(self, selection=None, **kwargs):
         keys, cmps = parse_selection(selection, **kwargs)
         sql, args = self.create_select_statement(keys, cmps, what='COUNT(*)')
-        con = self._connect()
-        self._initialize(con)
-        cur = con.cursor()
-        cur.execute(sql, args)
-        return cur.fetchone()[0]
+
+        with self.managed_connection() as con:
+            cur = con.cursor()
+            cur.execute(sql, args)
+            return cur.fetchone()[0]
 
     def analyse(self):
-        con = self._connect()
-        self._initialize(con)
-        con.execute('ANALYZE')
+        with self.managed_connection() as con:
+            con.execute('ANALYZE')
 
     @parallel_function
     @lock
     def delete(self, ids):
         if len(ids) == 0:
             return
-        con = self._connect()
-        self._delete(con.cursor(), ids,
-                     tables=self._get_external_table_names(db_con=con))
-        self._delete(con.cursor(), ids)
-        con.commit()
-        con.close()
+        table_names = self._get_external_table_names() + all_tables[::-1]
+        with self.managed_connection() as con:
+            self._delete(con.cursor(), ids,
+                         tables=table_names)
+        self.vacuum()
 
     def _delete(self, cur, ids, tables=None):
         tables = tables or all_tables[::-1]
@@ -770,6 +750,14 @@ class SQLite3Database(Database, object):
             cur.execute('DELETE FROM {} WHERE id in ({});'.
                         format(table, ', '.join([str(id) for id in ids])))
 
+    def vacuum(self):
+        if not self.type == 'db':
+            return
+
+        with self.managed_connection() as con:
+            con.commit()
+            con.cursor().execute("VACUUM")
+
     @property
     def metadata(self):
         if self._metadata is None:
@@ -779,38 +767,34 @@ class SQLite3Database(Database, object):
     @metadata.setter
     def metadata(self, dct):
         self._metadata = dct
-        con = self._connect()
-        self._initialize(con)
         md = json.dumps(dct)
-        cur = con.cursor()
-        cur.execute(
-            "SELECT COUNT(*) FROM information WHERE name='metadata'")
-
-        if cur.fetchone()[0]:
+        with self.managed_connection() as con:
+            cur = con.cursor()
             cur.execute(
-                "UPDATE information SET value=? WHERE name='metadata'", [md])
-        else:
-            cur.execute('INSERT INTO information VALUES (?, ?)',
-                        ('metadata', md))
-        con.commit()
+                "SELECT COUNT(*) FROM information WHERE name='metadata'")
+
+            if cur.fetchone()[0]:
+                cur.execute(
+                    "UPDATE information SET value=? WHERE name='metadata'",
+                    [md])
+            else:
+                cur.execute('INSERT INTO information VALUES (?, ?)',
+                            ('metadata', md))
 
     def _get_external_table_names(self, db_con=None):
         """Return a list with the external table names."""
-        con = db_con or self.connection or self._connect()
-        cur = con.cursor()
         sql = "SELECT value FROM information WHERE name='external_table_name'"
-        cur.execute(sql)
-        ext_tab_names = [x[0] for x in cur.fetchall()]
-
-        if self.connection is None and db_con is None:
-            con.close()
+        with self.managed_connection() as con:
+            cur = con.cursor()
+            cur.execute(sql)
+            ext_tab_names = [x[0] for x in cur.fetchall()]
         return ext_tab_names
 
     def _external_table_exists(self, name):
         """Return True if an external table name exists."""
         return name in self._get_external_table_names()
 
-    def _create_table_if_not_exists(self, name, dtype, db_con=None):
+    def _create_table_if_not_exists(self, name, dtype):
         """Create a new table if it does not exits.
 
         Arguments
@@ -820,49 +804,42 @@ class SQLite3Database(Database, object):
         dtype: str
             Datatype of the value field (typically REAL, INTEGER, TEXT etc.)
         """
-        if name in all_tables:
+
+        taken_names = set(all_tables + all_properties + self.columnnames)
+        if name in taken_names:
             raise ValueError("External table can not be any of {}"
-                             "".format(all_tables))
+                             "".format(taken_names))
 
         if self._external_table_exists(name):
             return
 
-        con = db_con or self.connection or self._connect()
-        cur = con.cursor()
         sql = "CREATE TABLE IF NOT EXISTS {} ".format(name)
         sql += "(key TEXT, value {}, id INTEGER, ".format(dtype)
         sql += "FOREIGN KEY (id) REFERENCES systems(id))"
-        cur.execute(sql)
-
-        sql = "INSERT INTO information VALUES (?, ?)"
-        # Insert an entry saying that there is a new external table
-        # present and an entry with the datatype
-        cur.execute(sql, ("external_table_name", name))
-        cur.execute(sql, (name + "_dtype", dtype))
-
-        if self.connection is None and db_con is None:
-            con.commit()
-            con.close()
+        sql2 = "INSERT INTO information VALUES (?, ?)"
+        with self.managed_connection() as con:
+            cur = con.cursor()
+            cur.execute(sql)
+            # Insert an entry saying that there is a new external table
+            # present and an entry with the datatype
+            cur.execute(sql2, ("external_table_name", name))
+            cur.execute(sql2, (name + "_dtype", dtype))
 
     def delete_external_table(self, name):
         """Delete an external table."""
         if not self._external_table_exists(name):
             return
 
-        con = self.connection or self._connect()
-        cur = con.cursor()
+        with self.managed_connection() as con:
+            cur = con.cursor()
 
-        sql = "DROP TABLE {}".format(name)
-        cur.execute(sql)
+            sql = "DROP TABLE {}".format(name)
+            cur.execute(sql)
 
-        sql = "DELETE FROM information WHERE value=?"
-        cur.execute(sql, (name,))
-        sql = "DELETE FROM information WHERE name=?"
-        cur.execute(sql, (name + "_dtype",))
-
-        if self.connection is None:
-            con.commit()
-            con.close()
+            sql = "DELETE FROM information WHERE value=?"
+            cur.execute(sql, (name,))
+            sql = "DELETE FROM information WHERE name=?"
+            cur.execute(sql, (name + "_dtype",))
 
     def _convert_to_recognized_types(self, value):
         """Convert Numpy types to python types."""
@@ -934,14 +911,13 @@ class SQLite3Database(Database, object):
 
     def _read_external_table(self, name, id):
         """Read row from external table."""
-        con = self.connection or self._connect()
-        cur = con.cursor()
-        cur.execute("SELECT * FROM {} WHERE id=?".format(name), (id,))
-        items = cur.fetchall()
-        dictionary = dict([(item[0], item[1]) for item in items])
 
-        if self.connection is None:
-            con.close()
+        with self.managed_connection() as con:
+            cur = con.cursor()
+            cur.execute("SELECT * FROM {} WHERE id=?".format(name), (id,))
+            items = cur.fetchall()
+            dictionary = dict([(item[0], item[1]) for item in items])
+
         return dictionary
 
 
