@@ -3,6 +3,7 @@ import numpy as np
 
 
 def bz_vertices(icell, dim=3):
+    """See https://xkcd.com/1421 ..."""
     from scipy.spatial import Voronoi
     icell = icell.copy()
     if dim < 3:
@@ -154,16 +155,23 @@ def bz_plot(cell, vectors=False, paths=None, points=None,
                     if num:
                         name = '{}_{{{}}}'.format(name, num)
                 if dimensions == 3:
-                    ax.text(x, y, z, '$' + name + '$',
-                            ha='center', va='bottom', color='r')
+                    ax.text(x, y, z, '$\\mathrm{' + name + '}$',
+                            ha='center', va='bottom', color='g')
                 elif dimensions == 2:
+                    ha_s = ['right', 'left', 'right']
+                    va_s = ['bottom', 'bottom', 'top']
+
+                    ha = ha_s[int(np.sign(x))]
+                    va = va_s[int(np.sign(y))]
                     if abs(z) < 1e-6:
-                        ax.text(x, y, '$' + name + '$',
-                                ha='center', va='bottom', color='r')
+                        ax.text(x, y, '$\\mathrm{' + name + '}$',
+                                ha=ha, va=va, color='g',
+                                zorder=5)
                 else:
                     if abs(y) < 1e-6 and abs(z) < 1e-6:
-                        ax.text(x, y, '$' + name + '$',
-                                ha='center', va='bottom', color='r')
+                        ax.text(x, y, '$\\mathrm{' + name + '}$',
+                                ha='center', va='bottom', color='g',
+                                zorder=5)
 
     if kpoints is not None:
         kw = {'c': 'b'}
@@ -173,9 +181,9 @@ def bz_plot(cell, vectors=False, paths=None, points=None,
             if dimensions == 3:
                 ax.scatter(p[0], p[1], p[2], **kw)
             elif dimensions == 2:
-                ax.scatter(p[0], p[1], c='b')
+                ax.scatter(p[0], p[1], zorder=4, **kw)
             else:
-                ax.scatter(p[0], 0, c='b')
+                ax.scatter(p[0], 0, zorder=4, **kw)
 
     ax.set_axis_off()
 
