@@ -3,41 +3,66 @@ import numpy as np
 from ase.utils import pbc2pbc
 
 
+max_it = 100000    # in practice this is not exceeded
+
+
+class CycleChecker:
+
+    def __init__(self, d):
+        assert d in [2, 3]
+
+        # worst case is the hexagonal cell in 2D and the fcc cell in 3D
+        n = {2: 6, 3: 12}[d]
+
+        # max cycle length is total number of primtive cell descriptions
+        max_cycle_length = np.prod([n - i for i in range(d)]) * np.prod(d)
+        self.visited = np.zeros((max_cycle_length, 3 * d), dtype=int)
+
+    def add_site(self, H):
+        # flatten array for simplicity
+        H = H.ravel()
+
+        # check if site exists
+        found = (self.visited == H).all(axis=1).any()
+
+        # shift all visited sites down and place current site at the top
+        self.visited = np.roll(self.visited, 1, axis=0)
+        self.visited[0] = H
+        return found
+
+
 def reduction_gauss(B, hu, hv):
     """Calculate a Gauss-reduced lattice basis (2D reduction)."""
+    cycle_checker = CycleChecker(d=2)
     u = hu @ B
     v = hv @ B
 
-    max_it = 100000    # in practice this is not exceeded
     for it in range(max_it):
-
         x = int(round(np.dot(u, v) / np.dot(u, u)))
         hu, hv = hv - x * hu, hu
-
         u = hu @ B
         v = hv @ B
-        if np.dot(u, u) >= np.dot(v, v):
+        site = np.array([hu, hv])
+        if np.dot(u, u) >= np.dot(v, v) or cycle_checker.add_site(site):
             return hv, hu
 
-    raise RuntimeError("Gaussian basis not found after %d iterations" % max_it)
+    raise RuntimeError(f"Gaussian basis not found after {max_it} iterations")
 
 
 def relevant_vectors_2D(u, v):
     cs = np.array([e for e in itertools.product([-1, 0, 1], repeat=2)])
-    vs = np.dot(cs, [u, v])
+    vs = cs @ [u, v]
     indices = np.argsort(np.linalg.norm(vs, axis=1))[:7]
     return vs[indices], cs[indices]
 
 
 def closest_vector(t0, u, v):
     t = t0
+    a = np.zeros(2, dtype=int)
     rs, cs = relevant_vectors_2D(u, v)
-    a = np.array([0, 0])
 
     dprev = float("inf")
-    max_it = 100000    # in practice this is not exceeded
     for it in range(max_it):
-
         ds = np.linalg.norm(rs + t, axis=1)
         index = np.argmin(ds)
         if index == 0 or ds[index] >= dprev:
@@ -49,47 +74,44 @@ def closest_vector(t0, u, v):
         a += kopt * cs[index]
         t = t0 + a[0] * u + a[1] * v
 
-    raise RuntimeError("Closest vector not found after %d iterations" % max_it)
+    raise RuntimeError(f"Closest vector not found after {max_it} iterations")
 
 
 def reduction_full(B):
     """Calculate a Minkowski-reduced lattice basis (3D reduction)."""
-    H = np.eye(3).astype(np.int)
+    cycle_checker = CycleChecker(d=3)
+    H = np.eye(3, dtype=int)
     norms = np.linalg.norm(B, axis=1)
 
-    max_it = 100000    # in practice this is not exceeded
     for it in range(max_it):
-
         # Sort vectors by norm
-        indices = np.argsort(norms)
-        H = H[indices]
+        H = H[np.argsort(norms, kind='merge')]
 
         # Gauss-reduce smallest two vectors
         hw = H[2]
         hu, hv = reduction_gauss(B, H[0], H[1])
-
         H = np.array([hu, hv, hw])
         R = H @ B
-        u, v, w = R
 
+        # Orthogonalize vectors using Gram-Schmidt
+        u, v, _ = R
         X = u / np.linalg.norm(u)
         Y = v - X * np.dot(v, X)
         Y /= np.linalg.norm(Y)
 
         # Find closest vector to last element of R
-        pu, pv, pw = np.dot(R, np.array([X, Y]).T)
+        pu, pv, pw = R @ np.array([X, Y]).T
         nb = closest_vector(pw, pu, pv)
-        hw = np.dot([nb[0], nb[1], 1], H)
 
         # Update basis
-        H = np.array([hu, hv, hw])
+        H[2] = [nb[0], nb[1], 1] @ H
         R = H @ B
 
-        norms = np.diag(np.dot(R, R.T))
-        if norms[2] >= norms[1] or (nb == 0).all():
+        norms = np.linalg.norm(R, axis=1)
+        if norms[2] >= norms[1] or cycle_checker.add_site(H):
             return R, H
 
-    raise RuntimeError("Reduced basis not found after %d iterations" % max_it)
+    raise RuntimeError(f"Reduced basis not found after {max_it} iterations")
 
 
 def minkowski_reduce(cell, pbc=True):
@@ -122,7 +144,7 @@ def minkowski_reduce(cell, pbc=True):
     pbc = pbc2pbc(pbc)
     dim = pbc.sum()
 
-    op = np.eye(3).astype(np.int)
+    op = np.eye(3, dtype=int)
     if dim == 2:
         perm = np.argsort(pbc, kind='merge')[::-1]    # stable sort
         pcell = cell[perm][:, perm]
