@@ -68,7 +68,7 @@ def read_aims(filename, apply_constraints=True):
                 fix_cart.append(FixCartesian(i, xyz))
             floatvect = float(inp[1]), float(inp[2]), float(inp[3])
             positions.append(floatvect)
-            magmoms.append(0.)
+            magmoms.append(0.0)
             symbols.append(inp[-1])
             i += 1
             xyz = np.array([0, 0, 0])
@@ -332,7 +332,7 @@ def write_aims(
 
 
 def get_sym_block(atoms):
-    """Get the symmetry block for the Parametric constraints in atoms.constraints"""
+    """Get symmetry block for Parametric constraints in atoms.constraints"""
     import numpy as np
 
     from ase.constraints import (
@@ -353,7 +353,8 @@ def get_sym_block(atoms):
 
             if np.any(atomic_param_constr[constr.indices] != ""):
                 warnings.warn(
-                    "multiple parametric constraints defined for the same atom, using the last one defined"
+                    "multiple parametric constraints defined for the same "
+                    "atom, using the last one defined"
                 )
 
             atomic_param_constr[constr.indices] = [
@@ -364,7 +365,8 @@ def get_sym_block(atoms):
 
             if np.any(lv_param_constr[constr.indices] != ""):
                 warnings.warn(
-                    "multiple parametric constraints defined for the same lattice vector, using the last one defined"
+                    "multiple parametric constraints defined for the same "
+                    "lattice vector, using the last one defined"
                 )
 
             lv_param_constr[constr.indices] = [
@@ -377,19 +379,22 @@ def get_sym_block(atoms):
     # Check Constraint Parameters
     if len(atomic_sym_params) != len(np.unique(atomic_sym_params)):
         warnings.warn(
-            "Some parameters were used across constraints, they will be combined in the aims calculations"
+            "Some parameters were used across constraints, they will be "
+            "combined in the aims calculations"
         )
         atomic_sym_params = np.unique(atomic_sym_params)
 
     if len(lv_sym_params) != len(np.unique(lv_sym_params)):
         warnings.warn(
-            "Some parameters were used across constraints, they will be combined in the aims calculations"
+            "Some parameters were used across constraints, they will be "
+            "combined in the aims calculations"
         )
         lv_sym_params = np.unique(lv_sym_params)
 
     if np.any(atomic_param_constr == ""):
         raise IOError(
-            "FHI-aims input files require all atoms have defined parametric constraints"
+            "FHI-aims input files require all atoms have defined parametric "
+            "constraints"
         )
 
     cell_inds = np.where(lv_param_constr == "")[0]
@@ -440,6 +445,28 @@ def read_energy(filename):
     return E
 
 
+def _parse_atoms(fd, n_atoms, molecular_dynamics=False):
+    """parse structure information from aims output to Atoms object"""
+    from ase import Atoms, Atom
+
+    next(fd)
+    atoms = Atoms()
+    for i in range(n_atoms):
+        inp = next(fd).split()
+        if "lattice_vector" in inp[0]:
+            cell = []
+            for i in range(3):
+                cell += [[float(inp[1]), float(inp[2]), float(inp[3])]]
+                inp = next(fd).split()
+            atoms.set_cell(cell)
+            inp = next(fd).split()
+        atoms.append(Atom(inp[4], (inp[1], inp[2], inp[3])))
+        if molecular_dynamics:
+            inp = next(fd).split()
+
+    return atoms
+
+
 def read_aims_output(filename, index=-1):
     """Import FHI-aims output files with all data available, i.e.
     relaxations, MD information, force information etc etc etc."""
@@ -494,30 +521,20 @@ def read_aims_output(filename, index=-1):
                     fix_cart.append(FixCartesian(ind, xyz))
                 else:
                     fix_cart[n].mask[xyz.index(1)] = 0
+
         if "Atomic structure:" in line and not molecular_dynamics:
             next(fd)
             atoms = Atoms()
-            for i in range(n_atoms):
+            for _ in range(n_atoms):
                 inp = next(fd).split()
                 atoms.append(Atom(inp[3], (inp[4], inp[5], inp[6])))
+
         if "Complete information for previous time-step:" in line:
             molecular_dynamics = True
+
         if "Updated atomic structure:" in line and not molecular_dynamics:
-            next(fd)
-            atoms = Atoms()
-            for i in range(n_atoms):
-                inp = next(fd).split()
-                if "lattice_vector" in inp[0]:
-                    cell = []
-                    for i in range(3):
-                        cell += [[float(inp[1]), float(inp[2]), float(inp[3])]]
-                        inp = next(fd).split()
-                    atoms.set_cell(cell)
-                    inp = next(fd).split()
-                atoms.append(Atom(inp[4], (inp[1], inp[2], inp[3])))
-                if molecular_dynamics:
-                    inp = next(fd).split()
-        if "Atomic structure (and velocities)" in line:
+            atoms = _parse_atoms(fd, n_atoms=n_atoms)
+        elif "Atomic structure (and velocities)" in line:
             next(fd)
             atoms = Atoms()
             velocities = []
@@ -534,6 +551,23 @@ def read_aims_output(filename, index=-1):
                 atoms.set_constraint(fix_cart)
             images.append(atoms)
 
+        # if we enter here, the SocketIO/PIMD Wrapper was used
+        elif (
+            "Atomic structure that "
+            "was used in the preceding time step of the wrapper"
+        ) in line:
+            # parse atoms and add calculator information, i.e., the energies
+            # and forces that were already collected
+            atoms = _parse_atoms(fd, n_atoms=n_atoms)
+            results = images[-1].calc.results
+            atoms.calc = SinglePointCalculator(atoms, **results)
+
+            # replace last image with updated atoms
+            images[-1] = atoms
+
+            # make sure `atoms` does not point to `images[-1` later on
+            atoms = atoms.copy()
+
         # FlK: add analytical stress and replace stress=None
         if "Analytical stress tensor - Symmetrized" in line:
             # scroll to significant lines
@@ -560,7 +594,7 @@ def read_aims_output(filename, index=-1):
                     calc = SinglePointCalculator(
                         atoms, energy=e, forces=f, stress=stress
                     )
-                images[-1].set_calculator(calc)
+                images[-1].calc = calc
             e = None
             f = None
 
@@ -570,7 +604,7 @@ def read_aims_output(filename, index=-1):
                 atoms.set_cell(cell)
                 atoms.pbc = True
             if not found_aims_calculator:
-                atoms.set_calculator(SinglePointCalculator(atoms, energy=e))
+                atoms.calc = SinglePointCalculator(atoms, energy=e)
             if not molecular_dynamics:
                 if len(fix):
                     atoms.set_constraint([FixAtoms(indices=fix)] + fix_cart)
@@ -580,7 +614,7 @@ def read_aims_output(filename, index=-1):
             e = None
             # if found_aims_calculator:
             # calc.set_results(images[-1])
-            # images[-1].set_calculator(calc)
+            # images[-1].calc = calc
 
         # FlK: add stress per atom
         if "Per atom stress (eV) used for heat flux calculation" in line:
@@ -601,7 +635,7 @@ def read_aims_output(filename, index=-1):
                 calc = SinglePointCalculator(
                     atoms, energy=e, forces=f, stress=stress, stresses=stresses
                 )
-                images[-1].set_calculator(calc)
+                images[-1].calc = calc
 
     fd.close()
     if molecular_dynamics:
