@@ -9,6 +9,7 @@ The "latin-1" encoding is required by the IUCR specification.
 import re
 import shlex
 import warnings
+from typing import Dict
 
 import numpy as np
 
@@ -16,7 +17,6 @@ from ase import Atoms
 from ase.parallel import paropen
 from ase.spacegroup import crystal
 from ase.spacegroup.spacegroup import spacegroup_from_data, Spacegroup
-from ase.utils import basestring
 from ase.data import atomic_numbers, atomic_masses
 from ase.io.cif_unicode import format_unicode, handle_subscripts
 
@@ -184,7 +184,7 @@ def parse_cif(fileobj, reader='ase'):
 def parse_cif_ase(fileobj):
     """Parse a CIF file using ase CIF parser"""
     blocks = []
-    if isinstance(fileobj, basestring):
+    if isinstance(fileobj, str):
         fileobj = open(fileobj, 'rb')
 
     data = fileobj.read()
@@ -213,7 +213,7 @@ def parse_cif_ase(fileobj):
 def parse_cif_pycodcif(fileobj):
     """Parse a CIF file using pycodcif CIF parser"""
     blocks = []
-    if not isinstance(fileobj, basestring):
+    if not isinstance(fileobj, str):
         fileobj = fileobj.name
 
     try:
@@ -223,7 +223,7 @@ def parse_cif_pycodcif(fileobj):
             'parse_cif_pycodcif requires pycodcif ' +
             '(http://wiki.crystallography.net/cod-tools/pycodcif/)')
 
-    data,_,_ = parse(fileobj)
+    data, _, _ = parse(fileobj)
 
     for datablock in data:
         tags = datablock['values']
@@ -351,7 +351,12 @@ def tags2atoms(tags, store_tags=False, primitive_cell=False,
         deuterium = False
 
     setting_name = None
-    if '_space_group_crystal_system' in tags:
+    if '_symmetry_space_group_setting' in tags:
+        setting = int(tags['_symmetry_space_group_setting'])
+        if setting < 1 or setting > 2:
+            raise ValueError('Spacegroup setting must be 1 or 2, not %d' %
+                             setting)
+    elif '_space_group_crystal_system' in tags:
         setting_name = tags['_space_group_crystal_system']
     elif '_symmetry_cell_setting' in tags:
         setting_name = tags['_symmetry_cell_setting']
@@ -492,20 +497,37 @@ def write_enc(fileobj, s):
     fileobj.write(s.encode("latin-1"))
 
 
-def write_cif(fileobj, images, format='default'):
-    """Write *images* to CIF file."""
-    if isinstance(fileobj, basestring):
+def write_cif(fileobj, images, cif_format='default',
+              wrap=True, labels=None, loop_keys={}) -> None:
+    """Write *images* to CIF file.
+
+    wrap: bool
+        Wrap atoms into unit cell.
+
+    labels: list
+        Use this list (shaped list[i_frame][i_atom] = string) for the '_atom_site_label'
+        section instead of automatically generating it from the element symbol.
+
+    loop_keys: dict
+        Add the information from this dictionary to the `loop_` section.
+        Keys are printed to the `loop_` section preceeded by '  _'. dict[key] should contain
+        the data printed for each atom, so it needs to have the setup
+        `dict[key][i_frame][i_atom] = string`. The strings are printed as
+        they are, so take care of formating. Information can be re-read using the `store_tags`
+        option of the cif reader.
+    """
+    if isinstance(fileobj, str):
         fileobj = paropen(fileobj, 'wb')
 
     if hasattr(images, 'get_positions'):
         images = [images]
 
-    for i, atoms in enumerate(images):
-        write_enc(fileobj, 'data_image%d\n' % i)
+    for i_frame, atoms in enumerate(images):
+        write_enc(fileobj, 'data_image%d\n' % i_frame)
 
         a, b, c, alpha, beta, gamma = atoms.get_cell_lengths_and_angles()
 
-        if format == 'mp':
+        if cif_format == 'mp':
 
             comp_name = atoms.get_chemical_formula(mode='reduce')
             sf = split_chem_form(comp_name)
@@ -546,7 +568,7 @@ def write_cif(fileobj, images, format='default'):
         # Is it a periodic system?
         coord_type = 'fract' if atoms.pbc.all() else 'Cartn'
 
-        if format == 'mp':
+        if cif_format == 'mp':
             write_enc(fileobj, '  _atom_site_type_symbol\n')
             write_enc(fileobj, '  _atom_site_label\n')
             write_enc(fileobj, '  _atom_site_symmetry_multiplicity\n')
@@ -565,19 +587,20 @@ def write_cif(fileobj, images, format='default'):
             write_enc(fileobj, '  _atom_site_type_symbol\n')
 
         if coord_type == 'fract':
-            coords = atoms.get_scaled_positions().tolist()
+            coords = atoms.get_scaled_positions(wrap).tolist()
         else:
-            coords = atoms.get_positions().tolist()
+            coords = atoms.get_positions(wrap).tolist()
         symbols = atoms.get_chemical_symbols()
         occupancies = [1 for i in range(len(symbols))]
 
-        # try to fetch occupancies // rely on the tag - occupancy mapping
+        # try to fetch occupancies // spacegroup_kinds - occupancy mapping
         try:
             occ_info = atoms.info['occupancy']
-            for i, tag in enumerate(atoms.get_tags()):
-                occupancies[i] = occ_info[tag][symbols[i]]
+            kinds = atoms.arrays['spacegroup_kinds']
+            for i, kind in enumerate(kinds):
+                occupancies[i] = occ_info[kind][symbols[i]]
                 # extend the positions array in case of mixed occupancy
-                for sym, occ in occ_info[tag].items():
+                for sym, occ in occ_info[kind].items():
                     if sym != symbols[i]:
                         symbols.append(sym)
                         coords.append(coords[i])
@@ -585,26 +608,44 @@ def write_cif(fileobj, images, format='default'):
         except KeyError:
             pass
 
-        no = {}
+        #can only do it now since length of atoms is not always equal to the number of entries
+        #do not move this up!
+        extra_data = ["" for i in range(len(symbols))]
+        for key in loop_keys:
+            extra_data = ["{}  {}".format(extra_data[i],loop_keys[key][i_frame][i]) for i in range(len(symbols))]
+            write_enc(fileobj, "  _{}\n".format(key))
 
-        for symbol, pos, occ in zip(symbols, coords, occupancies):
-            if symbol in no:
-                no[symbol] += 1
-            else:
-                no[symbol] = 1
-            if format == 'mp':
+
+
+        if labels:
+            included_labels = labels[i_frame]
+        else:
+            no: Dict[str, int] = {}
+            included_labels = []
+            for symbol in symbols:
+                if symbol in no:
+                    no[symbol] += 1
+                else:
+                    no[symbol] = 1
+                included_labels.append('%s%d' % (symbol, no[symbol]))
+
+        assert len(symbols) == len(coords) == len(occupancies) == len(included_labels) == len(extra_data)
+
+        for symbol, pos, occ, label, ext in zip(symbols, coords, occupancies, included_labels, extra_data):
+            if cif_format == 'mp':
                 write_enc(fileobj,
-                          '  %-2s  %4s  %4s  %7.5f  %7.5f  %7.5f  %6.1f\n' %
-                          (symbol, symbol + str(no[symbol]), 1,
-                           pos[0], pos[1], pos[2], occ))
+                          '  %-2s  %4s  %4s  %7.5f  %7.5f  %7.5f  %6.1f%s\n' %
+                          (symbol, label, 1,
+                           pos[0], pos[1], pos[2], occ, ext))
             else:
                 write_enc(fileobj,
-                          '  %-8s %6.4f %7.5f  %7.5f  %7.5f  %4s  %6.3f  %s\n'
-                          % ('%s%d' % (symbol, no[symbol]),
+                          '  %-8s %6.4f %7.5f  %7.5f  %7.5f  %4s  %6.3f  %-2s%s\n'
+                          % (label,
                              occ,
                              pos[0],
                              pos[1],
                              pos[2],
                              'Biso',
                              1.0,
-                             symbol))
+                             symbol,
+                             ext))
