@@ -12,12 +12,13 @@ __all__ = ['read_dlp4', 'write_dlp4']
 # dlp4 labels will be registered in atoms.arrays[DLP4_LABELS_KEY]
 DLP4_LABELS_KEY = 'dlp4_labels'
 
+
 def _get_frame_positions(f):
     """Get positions of frames in HISTORY file."""
-    #header line contains name of system
+    # header line contains name of system
     init_pos = f.tell()
     f.seek(0)
-    f.readline() #system name
+    rl = len(f.readline())  # system name, and record size
     items = f.readline().strip().split()
     if len(items) == 5:
         classic = False
@@ -26,9 +27,9 @@ def _get_frame_positions(f):
     else:
         raise RuntimeError("Cannot determine version of HISTORY file format.")
 
-    levcfg,imcon,natoms = [int(x) for x in items[0:3]]
+    levcfg, imcon, natoms = [int(x) for x in items[0:3]]
     if classic:
-        #we have to iterate over the entire file
+        # we have to iterate over the entire file
         startpos = f.tell()
         pos = []
         line = True
@@ -38,11 +39,12 @@ def _get_frame_positions(f):
                 pos.append(f.tell())
         f.seek(startpos)
     else:
-        nFrames = int(line[4])
-        pos = [(natoms * 2 + 1) * i + 2 for i in range(nFrames)]
+        nframes = int(items[3])
+        pos = [((natoms * (levcfg + 2) + 4) * i + 3) * rl for i in range(nframes)]
 
     f.seek(init_pos)
-    return levcfg,imcon,natoms,pos
+    return levcfg, imcon, natoms, pos
+
 
 def read_dlp_history(f, index=-1, symbols=None):
     """Read a HISTORY file.
@@ -51,9 +53,10 @@ def read_dlp_history(f, index=-1, symbols=None):
 
     *Index* can be integer or slice.
 
-    Provide a list of element strings to symbols to ignore naming from the HISTORY file.
+    Provide a list of element strings to symbols to ignore naming
+    from the HISTORY file.
     """
-    levcfg,imcon,natoms,pos = _get_frame_positions(f)
+    levcfg, imcon, natoms, pos = _get_frame_positions(f)
     if isscalar(index):
         selected = [pos[index]]
     else:
@@ -61,19 +64,20 @@ def read_dlp_history(f, index=-1, symbols=None):
 
     images = []
     for fpos in selected:
-        f.seek(fpos+1)
-        images.append(read_single_image(f, levcfg, imcon, natoms, is_trajectory=True, symbols=symbols))
+        f.seek(fpos + 1)
+        images.append(read_single_image(f, levcfg, imcon, natoms,
+                                        is_trajectory=True, symbols=symbols))
 
     return images
 
 
 def iread_dlp_history(f, symbols=None):
     """Generator version of read_history"""
-    levcfg,imcon,natoms,pos = _get_frame_positions(f)
+    levcfg, imcon, natoms, pos = _get_frame_positions(f)
     for p in pos:
-        f.seek(p+1)
-        yield read_single_image(f, levcfg, imcon, natoms, is_trajectory=True, symbols=symbols)
-
+        f.seek(p + 1)
+        yield read_single_image(f, levcfg, imcon, natoms, is_trajectory=True,
+                                symbols=symbols)
 
 
 def read_dlp4(f, symbols=None):
@@ -104,7 +108,8 @@ def read_dlp4(f, symbols=None):
             natoms = int(tokens[2])
         else:
             natoms = None
-        yield read_single_image(f, levcfg, imcon, natoms, is_trajectory, symbols)
+        yield read_single_image(f, levcfg, imcon, natoms, is_trajectory,
+                                symbols)
         line = f.readline()
 
 
@@ -150,13 +155,12 @@ def read_single_image(f, levcfg, imcon, natoms, is_trajectory, symbols=None):
         if not symbols:
             assert symbol in chemical_symbols
             sym.append(symbol)
-        #make sure label is not empty
+        # make sure label is not empty
         if label:
             labels.append(label)
         else:
             labels.append(line.split()[0])
 
-
         x, y, z = f.readline().split()[:3]
         positions.append([float(x), float(y), float(z)])
         if levcfg > 0:
@@ -166,8 +170,9 @@ def read_single_image(f, levcfg, imcon, natoms, is_trajectory, symbols=None):
             fx, fy, fz = f.readline().split()[:3]
             forces.append([float(fx), float(fy), float(fz)])
 
-    if symbols:
-        assert a+1 == len(symbols), ("Error, counter is at {:} but you gave {:} symbols".format(a+1,len(symbols)))
+    if symbols and a + 1 != len(symbols):
+        raise ValueError("Counter is at {} but you gave {} symbols"
+                         .format(a + 1, len(symbols)))
 
     if imcon == 0:
         pbc = False
@@ -188,7 +193,7 @@ def read_single_image(f, levcfg, imcon, natoms, is_trajectory, symbols=None):
     if levcfg > 0:
         atoms.set_velocities(velocities)
     if levcfg > 1:
-        atoms.set_calculator(SinglePointCalculator(atoms, forces=forces))
+        atoms.calc = SinglePointCalculator(atoms, forces=forces)
     return atoms
 
 
@@ -234,7 +239,7 @@ def write_dlp4(f, atoms, levcfg=0, title='CONFIG generated by ASE'):
         if labels is not None:
             sym += labels[i]
         f.write("{0:8s}{1:10d}\n{2:20.10f}{3:20.10f}{4:20.10f}\n".format(
-            sym, a.index+1, a.x, a.y, a.z))
+            sym, a.index + 1, a.x, a.y, a.z))
         if levcfg > 0:
             if vels is None:
                 f.write("{0:20.10f}{1:20.10f}{2:20.10f}\n".format(
