@@ -7,7 +7,6 @@ from ase.calculators.lammpsrun import Prism
 from ase.neighborlist import NeighborList
 from ase.data import atomic_masses, chemical_symbols
 from ase.io import read
-from ase.utils import basestring
 
 
 def twochar(name):
@@ -75,7 +74,7 @@ class OPLSff:
             self.read(fileobj)
 
     def read(self, fileobj, comments='#'):
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, str):
             fileobj = open(fileobj)
 
         def read_block(name, symlen, nvalues):
@@ -145,7 +144,7 @@ class OPLSff:
 
     def write_lammps_in(self):
         fileobj = self.prefix + '_in'
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, str):
             fileobj = open(fileobj, 'w')
         fileobj.write("""# LAMMPS relaxation (written by ASE)
 
@@ -174,7 +173,7 @@ restart         100000 test_relax
 
 min_style       fire
 minimize        1.0e-14 1.0e-5 100000 100000
-""")
+""")  # noqa: E501
         fileobj.close()
 
     def write_lammps_atoms(self, atoms, connectivities):
@@ -218,7 +217,7 @@ minimize        1.0e-14 1.0e-5 100000 100000
         else:
             molid = [1] * len(atoms)
         for i, r in enumerate(
-            p.positions_to_lammps_strs(atoms.get_positions())):
+                p.positions_to_lammps_strs(atoms.get_positions())):
             atype = atoms.types[tag[i]]
             if len(atype) < 2:
                 atype = atype + ' '
@@ -246,43 +245,37 @@ minimize        1.0e-14 1.0e-5 100000 100000
                            typ, cs))
 
         # bonds
-        if len(blist):
+        if blist:
             fileobj.write('\nBonds\n\n')
             for ib, bvals in enumerate(blist):
                 fileobj.write('%8d %6d %6d %6d ' %
                               (ib + 1, bvals[0] + 1, bvals[1] + 1,
                                bvals[2] + 1))
-                try:
+                if bvals[0] in btypes:
                     fileobj.write('# ' + btypes[bvals[0]])
-                except:
-                    pass
                 fileobj.write('\n')
 
         # angles
-        if len(alist):
+        if alist:
             fileobj.write('\nAngles\n\n')
             for ia, avals in enumerate(alist):
                 fileobj.write('%8d %6d %6d %6d %6d ' %
                               (ia + 1, avals[0] + 1,
                                avals[1] + 1, avals[2] + 1, avals[3] + 1))
-                try:
+                if avals[0] in atypes:
                     fileobj.write('# ' + atypes[avals[0]])
-                except:
-                    pass
                 fileobj.write('\n')
 
         # dihedrals
-        if len(dlist):
+        if dlist:
             fileobj.write('\nDihedrals\n\n')
             for i, dvals in enumerate(dlist):
                 fileobj.write('%8d %6d %6d %6d %6d %6d ' %
                               (i + 1, dvals[0] + 1,
                                dvals[1] + 1, dvals[2] + 1,
                                dvals[3] + 1, dvals[4] + 1))
-                try:
+                if dvals[0] in dtypes:
                     fileobj.write('# ' + dtypes[dvals[0]])
-                except:
-                    pass
                 fileobj.write('\n')
 
     def update_neighbor_list(self, atoms):
@@ -398,30 +391,30 @@ minimize        1.0e-14 1.0e-5 100000 100000
         dih_list = []
         dih_types = []
 
-        def append(name, i, j, k, l):
+        def append(name, i, j, k, L):
             if name not in dih_types:
                 dih_types.append(name)
             index = dih_types.index(name)
-            if (([index, i, j, k, l] not in dih_list) and
-                ([index, l, k, j, i] not in dih_list)):
-                dih_list.append([index, i, j, k, l])
+            if (([index, i, j, k, L] not in dih_list) and
+                ([index, L, k, j, i] not in dih_list)):
+                dih_list.append([index, i, j, k, L])
 
         for angle in ang_types:
-            l, i, j, k = angle
+            L, i, j, k = angle
             iname = types[tags[i]]
             jname = types[tags[j]]
             kname = types[tags[k]]
 
             # search for l-i-j-k
             indicesi, offsetsi = self.nl.get_neighbors(i)
-            for l, offsetl in zip(indicesi, offsetsi):
-                if l == j:
+            for L, offsetl in zip(indicesi, offsetsi):
+                if L == j:
                     continue  # avoid double count
-                lname = types[tags[l]]
+                lname = types[tags[L]]
                 cut = cutoffs.value(iname, lname)
                 if cut is None:
                     continue  # don't have it
-                dist = np.linalg.norm(atoms[i].position - atoms[l].position -
+                dist = np.linalg.norm(atoms[i].position - atoms[L].position -
                                       np.dot(offsetl, cell))
                 if dist > cut:
                     continue  # too far away
@@ -429,18 +422,18 @@ minimize        1.0e-14 1.0e-5 100000 100000
                                                       jname, kname)
                 if name is None:
                     continue  # don't have it
-                append(name, l, i, j, k)
+                append(name, L, i, j, k)
 
             # search for i-j-k-l
             indicesk, offsetsk = self.nl.get_neighbors(k)
-            for l, offsetl in zip(indicesk, offsetsk):
-                if l == j:
+            for L, offsetl in zip(indicesk, offsetsk):
+                if L == j:
                     continue  # avoid double count
-                lname = types[tags[l]]
+                lname = types[tags[L]]
                 cut = cutoffs.value(kname, lname)
                 if cut is None:
                     continue  # don't have it
-                dist = np.linalg.norm(atoms[k].position - atoms[l].position -
+                dist = np.linalg.norm(atoms[k].position - atoms[L].position -
                                       np.dot(offsetl, cell))
                 if dist > cut:
                     continue  # too far away
@@ -448,7 +441,7 @@ minimize        1.0e-14 1.0e-5 100000 100000
                                                       kname, lname)
                 if name is None:
                     continue  # don't have it
-                append(name, i, j, k, l)
+                append(name, i, j, k, L)
 
         return dih_types, dih_list
 
@@ -456,7 +449,7 @@ minimize        1.0e-14 1.0e-5 100000 100000
         """Write force field definitions for LAMMPS."""
 
         fileobj = self.prefix + '_opls'
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, str):
             fileobj = open(fileobj, 'w')
 
         fileobj.write('# OPLS potential\n')
@@ -612,7 +605,7 @@ class OPLSStructure(Atoms):
 
         update_types: update atom types from the masses
         """
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, str):
             fileobj = open(fileobj, 'r')
 
         lines = fileobj.readlines()
