@@ -16,14 +16,13 @@ import numpy as np
 from ase.atoms import Atoms
 from ase.parallel import paropen
 from ase.geometry import cellpar_to_cell
-from ase.utils import basestring
 from ase.io.espresso import label_to_symbol
 
 
 def read_atom_line(line_full):
     """
     Read atom line from pdb format
-    HETATM    1  H14 ORTE    0       6.301   0.693   1.919  1.00  0.00           H
+    HETATM    1  H14 ORTE    0       6.301   0.693   1.919  1.00  0.00        H
     """
 
     line = line_full.rstrip('\n')
@@ -34,7 +33,7 @@ def read_atom_line(line_full):
 
         altloc = line[16]
         resname = line[17:21]
-        # chainid = line[21]        # Not used 
+        # chainid = line[21]        # Not used
 
         resseq = int(line[22:26].split()[0])  # sequence identifier
         # icode = line[26]          # insertion code, not used
@@ -59,7 +58,8 @@ def read_atom_line(line_full):
         try:
             bfactor = float(line[60:66])
         except ValueError:
-            bfactor = 0.0  # The PDB use a default of zero if the data is missing
+            # The PDB use a default of zero if the data is missing
+            bfactor = 0.0
 
         # segid = line[72:76] # not used
         symbol = line[76:78].strip().upper()
@@ -69,10 +69,11 @@ def read_atom_line(line_full):
 
     return symbol, name, altloc, resname, coord, occupancy, bfactor, resseq
 
+
 def read_proteindatabank(fileobj, index=-1, read_arrays=True):
     """Read PDB files."""
 
-    if isinstance(fileobj, basestring):
+    if isinstance(fileobj, str):
         fileobj = open(fileobj)
 
     images = []
@@ -182,13 +183,12 @@ def read_proteindatabank(fileobj, index=-1, read_arrays=True):
 
 def write_proteindatabank(fileobj, images, write_arrays=True):
     """Write images to PDB-file."""
-    if isinstance(fileobj, basestring):
+    if isinstance(fileobj, str):
         fileobj = paropen(fileobj, 'w')
 
     if hasattr(images, 'get_positions'):
         images = [images]
 
-
     rotation = None
     if images[0].get_pbc().any():
         from ase.geometry import cell_to_cellpar, cellpar_to_cell
@@ -229,6 +229,6 @@ def write_proteindatabank(fileobj, images, write_arrays=True):
             x, y, z = p[a]
             occ = occupancy[a]
             bf = bfactor[a]
-            fileobj.write(format % (a % MAXNUM, symbols[a],
+            fileobj.write(format % ((a+1) % MAXNUM, symbols[a],
                                     x, y, z, occ, bf, symbols[a].upper()))
         fileobj.write('ENDMDL\n')
