@@ -10,14 +10,15 @@ import re
 import numpy as np
 
 import ase.units
+
 from ase import Atoms
-from ase.utils import basestring, reader, writer
+from ase.utils import reader, writer
 from ase.io.utils import ImageIterator, ImageChunk
 
-
-__all__ = ['read_vasp', 'read_vasp_out', 'iread_vasp_out',
-           'read_vasp_xdatcar', 'read_vasp_xml',
-           'write_vasp']
+__all__ = [
+    'read_vasp', 'read_vasp_out', 'iread_vasp_out', 'read_vasp_xdatcar',
+    'read_vasp_xml', 'write_vasp', 'write_vasp_xdatcar'
+]
 
 # Denotes end of Ionic step for OUTCAR reading
 _OUTCAR_SCF_DELIM = 'FREE ENERGIE OF THE ION-ELECTRON SYSTEM'
@@ -62,10 +63,11 @@ def atomtypes_outpot(posfname, numsyms):
 
     # First check files with exactly same name except POTCAR/OUTCAR instead
     # of POSCAR/CONTCAR.
-    fnames = [posfname.replace('POSCAR', 'POTCAR').replace('CONTCAR',
-                                                           'POTCAR')]
-    fnames.append(posfname.replace('POSCAR', 'OUTCAR').replace('CONTCAR',
-                                                               'OUTCAR'))
+    fnames = [
+        posfname.replace('POSCAR', 'POTCAR').replace('CONTCAR', 'POTCAR')
+    ]
+    fnames.append(
+        posfname.replace('POSCAR', 'OUTCAR').replace('CONTCAR', 'OUTCAR'))
     # Try the same but with compressed files
     fsc = []
     for fn in fnames:
@@ -120,7 +122,7 @@ def read_vasp(filename='CONTCAR'):
     from ase.constraints import FixAtoms, FixScaled
     from ase.data import chemical_symbols
 
-    f = filename
+    fd = filename
     # The first line is in principle a comment line, however in VASP
     # 4.x a common convention is to have it contain the atom symbols,
     # eg. "Ag Ge" in the same order as later in the file (and POTCAR
@@ -128,14 +130,14 @@ def read_vasp(filename='CONTCAR'):
     # is found on the fifth line. Thus we save the first line and use
     # it in case we later detect that we're reading a VASP 4.x format
     # file.
-    line1 = f.readline()
+    line1 = fd.readline()
 
-    lattice_constant = float(f.readline().split()[0])
+    lattice_constant = float(fd.readline().split()[0])
 
     # Now the lattice vectors
     a = []
     for ii in range(3):
-        s = f.readline().split()
+        s = fd.readline().split()
         floatvect = float(s[0]), float(s[1]), float(s[2])
         a.append(floatvect)
 
@@ -145,7 +147,7 @@ def read_vasp(filename='CONTCAR'):
     # in the first line
     # or in the POTCAR or OUTCAR file
     atom_symbols = []
-    numofatoms = f.readline().split()
+    numofatoms = fd.readline().split()
     # Check whether we have a VASP 4.x or 5.x format file. If the
     # format is 5.x, use the fifth line to provide information about
     # the atomic symbols.
@@ -155,7 +157,7 @@ def read_vasp(filename='CONTCAR'):
     except ValueError:
         vasp5 = True
         atomtypes = numofatoms
-        numofatoms = f.readline().split()
+        numofatoms = fd.readline().split()
 
     # check for comments in numofatoms line and get rid of them if necessary
     commentcheck = np.array(['!' in s for s in numofatoms])
@@ -164,7 +166,7 @@ def read_vasp(filename='CONTCAR'):
         numofatoms = numofatoms[:np.arange(len(numofatoms))[commentcheck][0]]
 
     if not vasp5:
-        # Split the comment line (first in the file) into words and 
+        # Split the comment line (first in the file) into words and
         # try to compose a list of chemical symbols
         from ase.formula import Formula
         import re
@@ -176,11 +178,11 @@ def read_vasp(filename='CONTCAR'):
             try:
                 atomtypes.extend(list(Formula(word_without_delims)))
             except ValueError:
-                #print(atomtype, e, 'is comment')
+                # print(atomtype, e, 'is comment')
                 pass
         # Now the list of chemical symbols atomtypes must be formed.
         # For example: atomtypes = ['Pd', 'C', 'O']
-        
+
         numsyms = len(numofatoms)
         if len(atomtypes) < numsyms:
             # First line in POSCAR/CONTCAR didn't contain enough symbols.
@@ -190,26 +192,26 @@ def read_vasp(filename='CONTCAR'):
             if len(atomtypes) == 1 and '_' in atomtypes[0]:
                 atomtypes = get_atomtypes_from_formula(atomtypes[0])
             else:
-                atomtypes = atomtypes_outpot(f.name, numsyms)
+                atomtypes = atomtypes_outpot(fd.name, numsyms)
         else:
             try:
                 for atype in atomtypes[:numsyms]:
                     if atype not in chemical_symbols:
                         raise KeyError
             except KeyError:
-                atomtypes = atomtypes_outpot(f.name, numsyms)
+                atomtypes = atomtypes_outpot(fd.name, numsyms)
 
     for i, num in enumerate(numofatoms):
         numofatoms[i] = int(num)
         [atom_symbols.append(atomtypes[i]) for na in range(numofatoms[i])]
 
     # Check if Selective dynamics is switched on
-    sdyn = f.readline()
+    sdyn = fd.readline()
     selective_dynamics = sdyn[0].lower() == 's'
 
     # Check if atom coordinates are cartesian or direct
     if selective_dynamics:
-        ac_type = f.readline()
+        ac_type = fd.readline()
     else:
         ac_type = sdyn
     cartesian = ac_type[0].lower() == 'c' or ac_type[0].lower() == 'k'
@@ -218,7 +220,7 @@ def read_vasp(filename='CONTCAR'):
     if selective_dynamics:
         selective_flags = np.empty((tot_natoms, 3), dtype=bool)
     for atom in range(tot_natoms):
-        ac = f.readline().split()
+        ac = fd.readline().split()
         atoms_pos[atom] = (float(ac[0]), float(ac[1]), float(ac[2]))
         if selective_dynamics:
             curflag = []
@@ -278,14 +280,14 @@ def _read_outcar_frame(lines, header_data):
 
     atoms = Atoms(symbols=symbols, pbc=True, constraint=constraints)
 
-    cl = _outcar_check_line     # Aliasing
+    cl = _outcar_check_line  # Aliasing
 
-    spinc = 0                   # Spin component
+    spinc = 0  # Spin component
     kpts = []
     forces = np.zeros((natoms, 3))
     positions = np.zeros((natoms, 3))
-    f_n = np.zeros(nbands)      # kpt occupations
-    eps_n = np.zeros(nbands)    # kpt eigenvalues
+    f_n = np.zeros(nbands)  # kpt occupations
+    eps_n = np.zeros(nbands)  # kpt eigenvalues
 
     # Parse each atoms object
     for n, line in enumerate(lines):
@@ -298,9 +300,11 @@ def _read_outcar_frame(lines, header_data):
             atoms.set_cell(cell)
         elif 'magnetization (x)' in line:
             # Magnetization in both collinear and non-collinear
-            nskip = 4           # Skip some lines
-            mag_x = [float(cl(lines[n + i + nskip]).split()[-1])
-                     for i in range(natoms)]
+            nskip = 4  # Skip some lines
+            mag_x = [
+                float(cl(lines[n + i + nskip]).split()[-1])
+                for i in range(natoms)
+            ]
 
         # XXX: !!!Uncomment these lines when non-collinear spin is supported!!!
         # Remember to check that format fits!
@@ -323,7 +327,7 @@ def _read_outcar_frame(lines, header_data):
                 if len(magmom) == 1:
                     # Collinear spin
                     magmom = float(magmom[0])
-                # XXX: !!!Uncomment these lines when non-collinear spin is supported!!!
+                # !Uncomment these lines when non-collinear spin is supported!
                 # Remember to check that format fits!
                 # else:
                 #     # Non-collinear spin
@@ -367,8 +371,8 @@ def _read_outcar_frame(lines, header_data):
                 parts = lines[n + i + nskip].split()
                 eps_n[i] = float(parts[1])
                 f_n[i] = float(parts[2])
-            kpts.append(SinglePointKPoint(w, spinc, ikpt,
-                                          eps_n=eps_n, f_n=f_n))
+            kpts.append(SinglePointKPoint(w, spinc, ikpt, eps_n=eps_n,
+                                          f_n=f_n))
         elif _OUTCAR_SCF_DELIM in line:
             # Last section before next ionic step
             nskip = 2
@@ -394,16 +398,15 @@ def _read_outcar_frame(lines, header_data):
                     # Collinear
                     magmoms = np.array(mag_x)
 
-            atoms.set_calculator(
-                SinglePointDFTCalculator(atoms,
-                                         energy=energy_zero,
-                                         free_energy=energy_free,
-                                         ibzkpts=ibzkpts,
-                                         forces=forces,
-                                         efermi=efermi,
-                                         magmom=magmom,
-                                         magmoms=magmoms,
-                                         stress=stress))
+            atoms.calc = SinglePointDFTCalculator(atoms,
+                                                  energy=energy_zero,
+                                                  free_energy=energy_free,
+                                                  ibzkpts=ibzkpts,
+                                                  forces=forces,
+                                                  efermi=efermi,
+                                                  magmom=magmom,
+                                                  magmoms=magmoms,
+                                                  stress=stress)
             atoms.calc.name = 'vasp'
             atoms.calc.kpts = kpts
     return atoms
@@ -431,7 +434,7 @@ def _read_outcar_header(fd):
     else:
         constraints = None
 
-    cl = _outcar_check_line     # Aliasing
+    cl = _outcar_check_line  # Aliasing
 
     species = []
     natoms = 0
@@ -473,14 +476,13 @@ def _read_outcar_header(fd):
 
         elif 'Iteration' in line:
             # Start of SCF cycle
-            header_data = dict(
-                natoms=natoms,
-                symbols=symbols,
-                constraints=constraints,
-                nkpts=nkpts,
-                nbands=nbands,
-                kpt_weights=np.array(kpt_weights),
-                ibzkpts=np.array(ibzkpts))
+            header_data = dict(natoms=natoms,
+                               symbols=symbols,
+                               constraints=constraints,
+                               nkpts=nkpts,
+                               nbands=nbands,
+                               kpt_weights=np.array(kpt_weights),
+                               ibzkpts=np.array(ibzkpts))
             return header_data
 
     # Incomplete OUTCAR, we can't determine atoms
@@ -522,10 +524,10 @@ def read_vasp_out(filename='OUTCAR', index=-1):
     Reads unitcell, atom positions, energies, and forces from the OUTCAR file
     and attempts to read constraints (if any) from CONTCAR/POSCAR, if present.
     """
-    f = filename
-    g = iread_vasp_out(f, index=index)
+    # "filename" is actually a file-descriptor thanks to @reader
+    g = iread_vasp_out(filename, index=index)
     # Code borrowed from formats.py:read
-    if isinstance(index, (slice, basestring)):
+    if isinstance(index, (slice, str)):
         # Return list of atoms
         return list(g)
     else:
@@ -544,37 +546,38 @@ def read_vasp_xdatcar(filename='XDATCAR', index=-1):
        Constraints ARE NOT stored in the XDATCAR, and as such, Atoms
        objects retrieved from the XDATCAR will not have constraints set.
     """
-    f = filename
+    fd = filename  # @reader decorator ensures this is a file descriptor
     images = list()
 
     cell = np.eye(3)
     atomic_formula = str()
 
     while True:
-        comment_line = f.readline()
+        comment_line = fd.readline()
         if "Direct configuration=" not in comment_line:
             try:
-                lattice_constant = float(f.readline())
+                lattice_constant = float(fd.readline())
             except Exception:
                 # XXX: When would this happen?
                 break
 
-            xx = [float(x) for x in f.readline().split()]
-            yy = [float(y) for y in f.readline().split()]
-            zz = [float(z) for z in f.readline().split()]
+            xx = [float(x) for x in fd.readline().split()]
+            yy = [float(y) for y in fd.readline().split()]
+            zz = [float(z) for z in fd.readline().split()]
             cell = np.array([xx, yy, zz]) * lattice_constant
 
-            symbols = f.readline().split()
-            numbers = [int(n) for n in f.readline().split()]
+            symbols = fd.readline().split()
+            numbers = [int(n) for n in fd.readline().split()]
             total = sum(numbers)
 
             atomic_formula = ''.join('{:s}{:d}'.format(sym, numbers[n])
                                      for n, sym in enumerate(symbols))
 
-            f.readline()
+            fd.readline()
 
-        coords = [np.array(f.readline().split(), np.float)
-                  for ii in range(total)]
+        coords = [
+            np.array(fd.readline().split(), np.float) for ii in range(total)
+        ]
 
         image = Atoms(atomic_formula, cell=cell, pbc=True)
         image.set_scaled_positions(np.array(coords))
@@ -592,17 +595,13 @@ def __get_xml_parameter(par):
     handling.
 
     """
-
     def to_bool(b):
         if b == 'T':
             return True
         else:
             return False
 
-    to_type = {'int': int,
-               'logical': to_bool,
-               'string': str,
-               'float': float}
+    to_type = {'int': int, 'logical': to_bool, 'string': str, 'float': float}
 
     text = par.text
     if text is None:
@@ -679,29 +678,29 @@ def read_vasp_xml(filename='vasprun.xml', index=-1):
 
                     natoms = len(species)
 
-                elif (elem.tag == 'structure' and
-                      elem.attrib.get('name') == 'initialpos'):
+                elif (elem.tag == 'structure'
+                      and elem.attrib.get('name') == 'initialpos'):
                     cell_init = np.zeros((3, 3), dtype=float)
 
-                    for i, v in enumerate(elem.find(
-                            "crystal/varray[@name='basis']")):
-                        cell_init[i] = np.array([
-                            float(val) for val in v.text.split()])
+                    for i, v in enumerate(
+                            elem.find("crystal/varray[@name='basis']")):
+                        cell_init[i] = np.array(
+                            [float(val) for val in v.text.split()])
 
                     scpos_init = np.zeros((natoms, 3), dtype=float)
 
-                    for i, v in enumerate(elem.find(
-                            "varray[@name='positions']")):
-                        scpos_init[i] = np.array([
-                            float(val) for val in v.text.split()])
+                    for i, v in enumerate(
+                            elem.find("varray[@name='positions']")):
+                        scpos_init[i] = np.array(
+                            [float(val) for val in v.text.split()])
 
                     constraints = []
                     fixed_indices = []
 
-                    for i, entry in enumerate(elem.findall(
-                            "varray[@name='selective']/v")):
-                        flags = (np.array(entry.text.split() ==
-                                          np.array(['F', 'F', 'F'])))
+                    for i, entry in enumerate(
+                            elem.findall("varray[@name='selective']/v")):
+                        flags = (np.array(
+                            entry.text.split() == np.array(['F', 'F', 'F'])))
                         if flags.all():
                             fixed_indices.append(i)
                         elif flags.any():
@@ -719,8 +718,8 @@ def read_vasp_xml(filename='vasprun.xml', index=-1):
                 elif elem.tag == 'dipole':
                     dblock = elem.find('v[@name="dipole"]')
                     if dblock is not None:
-                        dipole = np.array([float(val)
-                                           for val in dblock.text.split()])
+                        dipole = np.array(
+                            [float(val) for val in dblock.text.split()])
 
             elif event == 'start' and elem.tag == 'calculation':
                 calculation.append(elem)
@@ -728,7 +727,7 @@ def read_vasp_xml(filename='vasprun.xml', index=-1):
     except ET.ParseError as parse_error:
         if atoms_init is None:
             raise parse_error
-        if calculation[-1].find('energy') is None:
+        if calculation and calculation[-1].find('energy') is None:
             calculation = calculation[:-1]
         if not calculation:
             yield atoms_init
@@ -748,9 +747,10 @@ def read_vasp_xml(filename='vasprun.xml', index=-1):
         # e_0_energy - e_fr_energy from calculation/scstep/energy, then
         # apply that correction to e_fr_energy from calculation/energy.
         lastscf = step.findall('scstep/energy')[-1]
-        try:
-            lastdipole = step.findall('scstep/dipole')[-1]
-        except:
+        dipoles = step.findall('scstep/dipole')
+        if dipoles:
+            lastdipole = dipoles[-1]
+        else:
             lastdipole = None
 
         de = (float(lastscf.find('i[@name="e_0_energy"]').text) -
@@ -760,13 +760,13 @@ def read_vasp_xml(filename='vasprun.xml', index=-1):
         energy = free_energy + de
 
         cell = np.zeros((3, 3), dtype=float)
-        for i, vector in enumerate(step.find(
-                'structure/crystal/varray[@name="basis"]')):
+        for i, vector in enumerate(
+                step.find('structure/crystal/varray[@name="basis"]')):
             cell[i] = np.array([float(val) for val in vector.text.split()])
 
         scpos = np.zeros((natoms, 3), dtype=float)
-        for i, vector in enumerate(step.find(
-                'structure/varray[@name="positions"]')):
+        for i, vector in enumerate(
+                step.find('structure/varray[@name="positions"]')):
             scpos[i] = np.array([float(val) for val in vector.text.split()])
 
         forces = None
@@ -774,16 +774,16 @@ def read_vasp_xml(filename='vasprun.xml', index=-1):
         if fblocks is not None:
             forces = np.zeros((natoms, 3), dtype=float)
             for i, vector in enumerate(fblocks):
-                forces[i] = np.array([float(val)
-                                      for val in vector.text.split()])
+                forces[i] = np.array(
+                    [float(val) for val in vector.text.split()])
 
         stress = None
         sblocks = step.find('varray[@name="stress"]')
         if sblocks is not None:
             stress = np.zeros((3, 3), dtype=float)
             for i, vector in enumerate(sblocks):
-                stress[i] = np.array([float(val)
-                                      for val in vector.text.split()])
+                stress[i] = np.array(
+                    [float(val) for val in vector.text.split()])
             stress *= -0.1 * GPa
             stress = stress.reshape(9)[[0, 4, 8, 5, 2, 1]]
 
@@ -818,19 +818,23 @@ def read_vasp_xml(filename='vasprun.xml', index=-1):
                         f_n[j] = float(val[1])
                     if len(kblocks) == 1:
                         f_n *= 2
-                    kpoints.append(SinglePointKPoint(kpt_weights[ikpt - 1],
-                                                     spin, ikpt, eps_n, f_n))
+                    kpoints.append(
+                        SinglePointKPoint(kpt_weights[ikpt - 1], spin, ikpt,
+                                          eps_n, f_n))
         if len(kpoints) == 0:
             kpoints = None
 
         atoms = atoms_init.copy()
         atoms.set_cell(cell)
         atoms.set_scaled_positions(scpos)
-        atoms.set_calculator(
-            SinglePointDFTCalculator(atoms, energy=energy, forces=forces,
-                                     stress=stress, free_energy=free_energy,
-                                     ibzkpts=ibz_kpts,
-                                     efermi=efermi, dipole=dipole))
+        atoms.calc = SinglePointDFTCalculator(atoms,
+                                              energy=energy,
+                                              forces=forces,
+                                              stress=stress,
+                                              free_energy=free_energy,
+                                              ibzkpts=ibz_kpts,
+                                              efermi=efermi,
+                                              dipole=dipole)
         atoms.calc.name = 'vasp'
         atoms.calc.kpts = kpoints
         atoms.calc.parameters = parameters
@@ -838,20 +842,137 @@ def read_vasp_xml(filename='vasprun.xml', index=-1):
 
 
 @writer
-def write_vasp(filename, atoms, label='', direct=False, sort=None,
-               symbol_count=None, long_format=True, vasp5=False,
+def write_vasp_xdatcar(fd, images, label=None):
+    """Write VASP MD trajectory (XDATCAR) file
+
+    Only Vasp 5 format is supported (for consistency with read_vasp_xdatcar)
+
+    Args:
+        fd (str, fp): Output file
+        images (iterable of Atoms): Atoms images to write. These must have
+            consistent atom order and lattice vectors - this will not be
+            checked.
+        label (str): Text for first line of file. If empty, default to list of
+            elements.
+
+    """
+
+    images = iter(images)
+    image = next(images)
+
+    if not isinstance(image, Atoms):
+        raise TypeError("images should be a sequence of Atoms objects.")
+
+    symbol_count = _symbol_count_from_symbols(image.get_chemical_symbols())
+
+    if label is None:
+        label = ' '.join([s for s, _ in symbol_count])
+    fd.write(label + '\n')
+
+    # Not using lattice constants, set it to 1
+    fd.write('           1\n')
+
+    # Lattice vectors; use first image
+    float_string = '{:11.6f}'
+    for row_i in range(3):
+        fd.write('  ')
+        fd.write(' '.join(float_string.format(x) for x in image.cell[row_i]))
+        fd.write('\n')
+
+    _write_symbol_count(fd, symbol_count)
+    _write_xdatcar_config(fd, image, index=1)
+    for i, image in enumerate(images):
+        # Index is off by 2: 1-indexed file vs 0-indexed Python;
+        # and we already wrote the first block.
+        _write_xdatcar_config(fd, image, i + 2)
+
+
+def _write_xdatcar_config(fd, atoms, index):
+    """Write a block of positions for XDATCAR file
+
+    Args:
+        fd (fd): writeable Python file descriptor
+        atoms (ase.Atoms): Atoms to write
+        index (int): configuration number written to block header
+
+    """
+    fd.write("Direct configuration={:6d}\n".format(index))
+    float_string = '{:11.8f}'
+    scaled_positions = atoms.get_scaled_positions()
+    for row in scaled_positions:
+        fd.write(' ')
+        fd.write(' '.join([float_string.format(x) for x in row]))
+        fd.write('\n')
+
+
+def _symbol_count_from_symbols(symbols):
+    """Reduce list of chemical symbols into compact VASP notation
+
+    args:
+        symbols (iterable of str)
+
+    returns:
+        list of pairs [(el1, c1), (el2, c2), ...]
+    """
+    sc = []
+    psym = symbols[0]
+    count = 0
+    for sym in symbols:
+        if sym != psym:
+            sc.append((psym, count))
+            psym = sym
+            count = 1
+        else:
+            count += 1
+    sc.append((psym, count))
+    return sc
+
+
+def _write_symbol_count(fd, sc, vasp5=True):
+    """Write the symbols and numbers block for POSCAR or XDATCAR
+
+    Args:
+        f (fd): Descriptor for writable file
+        sc (list of 2-tuple): list of paired elements and counts
+        vasp5 (bool): if False, omit symbols and only write counts
+
+    e.g. if sc is [(Sn, 4), (S, 6)] then write::
+
+      Sn   S
+       4   6
+
+    """
+    if vasp5:
+        for sym, _ in sc:
+            fd.write(' {:3s}'.format(sym))
+        fd.write('\n')
+
+    for _, count in sc:
+        fd.write(' {:3d}'.format(count))
+    fd.write('\n')
+
+
+@writer
+def write_vasp(filename,
+               atoms,
+               label=None,
+               direct=False,
+               sort=None,
+               symbol_count=None,
+               long_format=True,
+               vasp5=False,
                ignore_constraints=False):
     """Method to write VASP position (POSCAR/CONTCAR) files.
 
     Writes label, scalefactor, unitcell, # of various kinds of atoms,
     positions in cartesian or scaled coordinates (Direct), and constraints
-    to file. Cartesian coordiantes is default and default label is the
+    to file. Cartesian coordinates is default and default label is the
     atomic species, e.g. 'C N H Cu'.
     """
 
     from ase.constraints import FixAtoms, FixScaled, FixedPlane, FixedLine
 
-    f = filename
+    fd = filename  # @writer decorator ensures this arg is a file descriptor
 
     if isinstance(atoms, (list, tuple)):
         if len(atoms) > 1:
@@ -911,58 +1032,40 @@ def write_vasp(filename, atoms, label='', direct=False, sort=None,
     if symbol_count:
         sc = symbol_count
     else:
-        sc = []
-        psym = symbols[0]
-        count = 0
-        for sym in symbols:
-            if sym != psym:
-                sc.append((psym, count))
-                psym = sym
-                count = 1
-            else:
-                count += 1
-        sc.append((psym, count))
+        sc = _symbol_count_from_symbols(symbols)
 
     # Create the label
-    if label == '':
+    if label is None:
+        label = ''
         for sym, c in sc:
             label += '%2s ' % sym
-    f.write(label + '\n')
+    fd.write(label + '\n')
 
     # Write unitcell in real coordinates and adapt to VASP convention
     # for unit cell
     # ase Atoms doesn't store the lattice constant separately, so always
     # write 1.0.
-    f.write('%19.16f\n' % 1.0)
+    fd.write('%19.16f\n' % 1.0)
     if long_format:
         latt_form = ' %21.16f'
     else:
         latt_form = ' %11.6f'
     for vec in atoms.get_cell():
-        f.write(' ')
+        fd.write(' ')
         for el in vec:
-            f.write(latt_form % el)
-        f.write('\n')
+            fd.write(latt_form % el)
+        fd.write('\n')
 
-    # If we're writing a VASP 5.x format POSCAR file, write out the
-    # atomic symbols
-    if vasp5:
-        for sym, c in sc:
-            f.write(' %3s' % sym)
-        f.write('\n')
-
-    # Numbers of each atom
-    for sym, count in sc:
-        f.write(' %3i' % count)
-    f.write('\n')
+    # Write out symbols (if VASP 5.x) and counts of atoms
+    _write_symbol_count(fd, sc, vasp5=vasp5)
 
     if constraints:
-        f.write('Selective dynamics\n')
+        fd.write('Selective dynamics\n')
 
     if direct:
-        f.write('Direct\n')
+        fd.write('Direct\n')
     else:
-        f.write('Cartesian\n')
+        fd.write('Cartesian\n')
 
     if long_format:
         cform = ' %19.16f'
@@ -970,12 +1073,12 @@ def write_vasp(filename, atoms, label='', direct=False, sort=None,
         cform = ' %9.6f'
     for iatom, atom in enumerate(coord):
         for dcoord in atom:
-            f.write(cform % dcoord)
+            fd.write(cform % dcoord)
         if constraints:
             for flag in sflags[iatom]:
                 if flag:
                     s = 'F'
                 else:
                     s = 'T'
-                f.write('%4s' % s)
-        f.write('\n')
+                fd.write('%4s' % s)
+        fd.write('\n')
