@@ -1,4 +1,3 @@
-# flake8: noqa
 from abc import abstractmethod, ABC
 import functools
 import warnings
@@ -92,12 +91,13 @@ class BravaisLattice(ABC):
         cell = self._cell(**self._parameters)
         return Cell(cell)
 
-    def get_transformation(self, cell):
+    def get_transformation(self, cell, eps=1e-8):
         # Get transformation matrix relating input cell to canonical cell
         T = cell.dot(np.linalg.pinv(self.tocell()))
         msg = 'This transformation changes the length/area/volume of the cell'
         assert np.isclose(np.abs(np.linalg.det(T[:self.ndim,
-                                                 :self.ndim])), 1), msg
+                                                 :self.ndim])), 1,
+                          atol=eps), msg
         return T
 
     def cellpar(self):
@@ -128,7 +128,6 @@ class BravaisLattice(ABC):
         assert len(labels) == 1  # list of lists
         return labels[0]
 
-
     def get_special_points_array(self):
         """Return all special points for this lattice as an array.
 
@@ -173,7 +172,8 @@ class BravaisLattice(ABC):
         See :meth:`ase.cell.Cell.bandpath` for description of parameters.
 
         >>> BCT(3, 5).bandpath()
-        BandPath(path='GXYSGZS1NPY1Z,XP', cell=[3x3], special_points={GNPSS1XYY1Z}, kpts=[51x3])
+        BandPath(path='GXYSGZS1NPY1Z,XP', cell=[3x3], \
+special_points={GNPSS1XYY1Z}, kpts=[51x3])
 
         .. note:: This produces the standard band path following AFlow
            conventions.  If your cell does not follow this convention,
@@ -188,8 +188,9 @@ class BravaisLattice(ABC):
             path = self._variant.special_path
         elif not isinstance(path, str):
             from ase.dft.kpoints import resolve_custom_points
-            special_points = dict(special_points)
-            path = resolve_custom_points(path, special_points, self._eps)
+            path, special_points = resolve_custom_points(path,
+                                                         special_points,
+                                                         self._eps)
 
         cell = self.tocell()
         if transformation is not None:
@@ -347,6 +348,7 @@ class UnconventionalLattice(ValueError):
 
 class Cubic(BravaisLattice):
     """Abstract class for cubic lattices."""
+
     def __init__(self, a):
         BravaisLattice.__init__(self, a=a)
 
@@ -407,25 +409,25 @@ class BCT(BravaisLattice):
 
         if variant.name == 'BCT1':
             eta = .25 * (1 + c2 / a2)
-            points = [[0,0,0],
+            points = [[0, 0, 0],
                       [-.5, .5, .5],
-                      [0.,.5,0.],
+                      [0., .5, 0.],
                       [.25, .25, .25],
-                      [0.,0.,.5],
-                      [eta,eta,-eta],
-                      [-eta,1-eta,eta]]
+                      [0., 0., .5],
+                      [eta, eta, -eta],
+                      [-eta, 1 - eta, eta]]
         else:
             eta = .25 * (1 + a2 / c2)  # Not same eta as BCT1!
             zeta = 0.5 * a2 / c2
-            points = [[0.,.0,0.],
-                      [0.,.5,0.],
-                      [.25,.25,.25],
-                      [-eta,eta,eta],
-                      [eta,1-eta,-eta],
-                      [0.,0.,.5],
-                      [-zeta,zeta,.5],
-                      [.5,.5,-zeta],
-                      [.5,.5,-.5]]
+            points = [[0., .0, 0.],
+                      [0., .5, 0.],
+                      [.25, .25, .25],
+                      [-eta, eta, eta],
+                      [eta, 1 - eta, -eta],
+                      [0., 0., .5],
+                      [-zeta, zeta, .5],
+                      [.5, .5, -zeta],
+                      [.5, .5, -.5]]
         return points
 
 
@@ -437,6 +439,7 @@ def check_orc(a, b, c):
 
 class Orthorhombic(BravaisLattice):
     """Abstract class for orthorhombic types."""
+
     def __init__(self, a, b, c):
         check_orc(a, b, c)
         BravaisLattice.__init__(self, a=a, b=b, c=c)
@@ -486,14 +489,14 @@ class ORCF(Orthorhombic):
             delta = 0.25 * (1 + b2 / a2 - b2 / c2)
             eta = xminus
 
-            points = [[0,0,0],
-                      [.5, .5-eta, 1-eta],
-                      [.5, .5+eta, eta],
-                      [.5-delta, .5, 1-delta],
-                      [.5+delta, .5, delta],
+            points = [[0, 0, 0],
+                      [.5, .5 - eta, 1 - eta],
+                      [.5, .5 + eta, eta],
+                      [.5 - delta, .5, 1 - delta],
+                      [.5 + delta, .5, delta],
                       [.5, .5, .5],
-                      [1-phi, .5-phi, .5],
-                      [phi, .5+phi, .5],
+                      [1 - phi, .5 - phi, .5],
+                      [phi, .5 + phi, .5],
                       [0., .5, .5],
                       [.5, 0., .5],
                       [.5, .5, 0.]]
@@ -524,19 +527,19 @@ class ORCI(Orthorhombic):
         delta = .25 * (b2 - a2) / c2
         mu = .25 * (a2 + b2) / c2
 
-        points = [[0.,0.,0.],
-                  [-mu,mu,.5-delta],
-                  [mu, -mu, .5+delta],
-                  [.5-delta, .5+delta, -mu],
-                  [0,.5,0],
-                  [.5,0,0],
-                  [0.,0.,.5],
-                  [.25,.25,.25],
+        points = [[0., 0., 0.],
+                  [-mu, mu, .5 - delta],
+                  [mu, -mu, .5 + delta],
+                  [.5 - delta, .5 + delta, -mu],
+                  [0, .5, 0],
+                  [.5, 0, 0],
+                  [0., 0., .5],
+                  [.25, .25, .25],
                   [-zeta, zeta, zeta],
                   [zeta, 1 - zeta, -zeta],
                   [eta, -eta, eta],
                   [1 - eta, eta, -eta],
-                  [.5,.5,-.5]]
+                  [.5, .5, -.5]]
         return points
 
 
@@ -547,8 +550,7 @@ class ORCC(BravaisLattice):
     def __init__(self, a, b, c):
         # ORCC is the only ORCx lattice with a<b and not a<b<c
         if a >= b:
-            raise UnconventionalLattice('Expected a < b, got {}, {}'
-                                        .format(a, b, c))
+            raise UnconventionalLattice(f'Expected a < b, got a={a}, b={b}')
         BravaisLattice.__init__(self, a=a, b=b, c=c)
 
     def _cell(self, a, b, c):
@@ -557,16 +559,16 @@ class ORCC(BravaisLattice):
 
     def _special_points(self, a, b, c, variant):
         zeta = .25 * (1 + a * a / (b * b))
-        points = [[0,0,0],
-                  [zeta,zeta,.5],
-                  [-zeta,1-zeta,.5],
-                  [0,.5,.5],
-                  [0,.5,0],
-                  [-.5,.5,.5],
-                  [zeta,zeta,0],
-                  [-zeta,1-zeta,0],
-                  [-.5,.5,0],
-                  [0,0,.5]]
+        points = [[0, 0, 0],
+                  [zeta, zeta, .5],
+                  [-zeta, 1 - zeta, .5],
+                  [0, .5, .5],
+                  [0, .5, 0],
+                  [-.5, .5, .5],
+                  [zeta, zeta, 0],
+                  [-zeta, 1 - zeta, 0],
+                  [-.5, .5, 0],
+                  [0, 0, .5]]
         return points
 
 
@@ -614,29 +616,29 @@ class RHL(BravaisLattice):
             cosa = np.cos(alpha * _degrees)
             eta = (1 + 4 * cosa) / (2 + 4 * cosa)
             nu = .75 - 0.5 * eta
-            points = [[0,0,0],
-                      [eta,.5,1-eta],
+            points = [[0, 0, 0],
+                      [eta, .5, 1 - eta],
                       [.5, 1 - eta, eta - 1],
-                      [.5,.5,0],
-                      [.5,0,0],
-                      [0,0,-.5],
-                      [eta,nu,nu],
-                      [1-nu,1-nu,1-eta],
-                      [nu,nu,eta-1],
-                      [1-nu,nu,0],
-                      [nu,0,-nu],
-                      [.5,.5,.5]]
+                      [.5, .5, 0],
+                      [.5, 0, 0],
+                      [0, 0, -.5],
+                      [eta, nu, nu],
+                      [1 - nu, 1 - nu, 1 - eta],
+                      [nu, nu, eta - 1],
+                      [1 - nu, nu, 0],
+                      [nu, 0, -nu],
+                      [.5, .5, .5]]
         else:
             eta = 1 / (2 * np.tan(alpha * _degrees / 2)**2)
             nu = .75 - 0.5 * eta
-            points = [[0,0,0],
-                      [.5,-.5,0],
-                      [.5,0,0],
-                      [1-nu,-nu,1-nu],
-                      [nu,nu-1,nu-1],
-                      [eta,eta,eta],
-                      [1-eta,-eta,-eta],
-                      [.5,-.5,.5]]
+            points = [[0, 0, 0],
+                      [.5, -.5, 0],
+                      [.5, 0, 0],
+                      [1 - nu, -nu, 1 - nu],
+                      [nu, nu - 1, nu - 1],
+                      [eta, eta, eta],
+                      [1 - eta, -eta, -eta],
+                      [.5, -.5, .5]]
         return points
 
 
@@ -665,22 +667,22 @@ class MCL(BravaisLattice):
         eta = (1 - b * cosa / c) / (2 * np.sin(alpha * _degrees)**2)
         nu = .5 - eta * c * cosa / b
 
-        points = [[0,0,0],
-                  [.5,.5,0],
-                  [0,.5,.5],
-                  [.5,0,.5],
-                  [.5,0,-.5],
-                  [.5,.5,.5],
-                  [0,eta,1-nu],
-                  [0,1-eta,nu],
-                  [0,eta,-nu],
-                  [.5,eta,1-nu],
-                  [.5,1-eta,nu],
-                  [.5,eta,-nu],
-                  [0,.5,0],
-                  [0,0,.5],
-                  [0,0,-.5],
-                  [.5,0,0]]
+        points = [[0, 0, 0],
+                  [.5, .5, 0],
+                  [0, .5, .5],
+                  [.5, 0, .5],
+                  [.5, 0, -.5],
+                  [.5, .5, .5],
+                  [0, eta, 1 - nu],
+                  [0, 1 - eta, nu],
+                  [0, eta, -nu],
+                  [.5, eta, 1 - nu],
+                  [.5, 1 - eta, nu],
+                  [.5, eta, -nu],
+                  [0, .5, 0],
+                  [0, 0, .5],
+                  [0, 0, -.5],
+                  [.5, 0, 0]]
         return points
 
     def _variant_name(self, a, b, c, alpha):
@@ -711,7 +713,7 @@ class MCLC(BravaisLattice):
                          [0, c * np.cos(alpha), c * np.sin(alpha)]])
 
     def _variant_name(self, a, b, c, alpha):
-        #from ase.geometry.cell import mclc
+        # from ase.geometry.cell import mclc
         # okay, this is a bit hacky
 
         # We need the same parameters here as when determining the points.
@@ -763,48 +765,48 @@ class MCLC(BravaisLattice):
             psi = .75 - a2 / (4 * b2 * sina * sina)
             phi = psi + (.75 - psi) * b * cosa / c
 
-            points = [[0,0,0],
-                      [.5,0,0],
-                      [0,-.5,0],
-                      [1-zeta,1-zeta,1-eta],
-                      [zeta,zeta,eta],
-                      [-zeta,-zeta,1-eta],
-                      [1-zeta,-zeta,1-eta],
-                      [phi,1-phi,.5],
-                      [1-phi,phi-1,.5],
-                      [.5,.5,.5],
-                      [.5,0,.5],
-                      [1-psi,psi-1,0],
-                      [psi,1-psi,0],
-                      [psi-1,-psi,0],
-                      [.5,.5,0],
-                      [-.5,-.5,0],
-                      [0,0,.5]]
+            points = [[0, 0, 0],
+                      [.5, 0, 0],
+                      [0, -.5, 0],
+                      [1 - zeta, 1 - zeta, 1 - eta],
+                      [zeta, zeta, eta],
+                      [-zeta, -zeta, 1 - eta],
+                      [1 - zeta, -zeta, 1 - eta],
+                      [phi, 1 - phi, .5],
+                      [1 - phi, phi - 1, .5],
+                      [.5, .5, .5],
+                      [.5, 0, .5],
+                      [1 - psi, psi - 1, 0],
+                      [psi, 1 - psi, 0],
+                      [psi - 1, -psi, 0],
+                      [.5, .5, 0],
+                      [-.5, -.5, 0],
+                      [0, 0, .5]]
         elif variant == 3 or variant == 4:
             mu = .25 * (1 + b2 / a2)
-            delta = b * c * cosa / (2  * a2)
+            delta = b * c * cosa / (2 * a2)
             zeta = mu - 0.25 + (1 - b * cosa / c) / (4 * sina2)
             eta = 0.5 + 2 * zeta * c * cosa / b
             phi = 1 + zeta - 2 * mu
             psi = eta - 2 * delta
 
-            points = [[0,0,0],
-                      [1-phi,1-phi,1-psi],
-                      [phi,phi-1,psi],
-                      [1-phi,-phi,1-psi],
-                      [zeta,zeta,eta],
-                      [1-zeta,-zeta,1-eta],
-                      [-zeta,-zeta,1-eta],
-                      [.5,-.5,.5],
-                      [.5,0,.5],
-                      [.5,0,0],
-                      [0,-.5,0],
-                      [.5,-.5,0],
-                      [mu,mu,delta],
-                      [1-mu,-mu,-delta],
-                      [-mu,-mu,-delta],
-                      [mu,mu-1,delta],
-                      [0,0,.5]]
+            points = [[0, 0, 0],
+                      [1 - phi, 1 - phi, 1 - psi],
+                      [phi, phi - 1, psi],
+                      [1 - phi, -phi, 1 - psi],
+                      [zeta, zeta, eta],
+                      [1 - zeta, -zeta, 1 - eta],
+                      [-zeta, -zeta, 1 - eta],
+                      [.5, -.5, .5],
+                      [.5, 0, .5],
+                      [.5, 0, 0],
+                      [0, -.5, 0],
+                      [.5, -.5, 0],
+                      [mu, mu, delta],
+                      [1 - mu, -mu, -delta],
+                      [-mu, -mu, -delta],
+                      [mu, mu - 1, delta],
+                      [0, 0, .5]]
         elif variant == 5:
             zeta = .25 * (b2 / a2 + (1 - b * cosa / c) / sina2)
             eta = 0.5 + 2 * zeta * c * cosa / b
@@ -814,25 +816,25 @@ class MCLC(BravaisLattice):
             delta = zeta * c * cosa / b + omega / 2 - .25
             rho = 1 - zeta * a2 / b2
 
-            points = [[0,0,0],
-                      [nu,nu,omega],
-                      [1-nu,1-nu,1-omega],
-                      [nu,nu-1,omega],
-                      [zeta,zeta,eta],
-                      [1-zeta,-zeta,1-eta],
-                      [-zeta,-zeta,1-eta],
-                      [rho,1-rho,.5],
-                      [1-rho,rho-1,.5],
-                      [.5,.5,.5],
-                      [.5,0,.5],
-                      [.5,0,0],
-                      [0,-.5,0],
-                      [.5,-.5,0],
-                      [mu,mu,delta],
-                      [1-mu,-mu,-delta],
-                      [-mu,-mu,-delta],
-                      [mu,mu-1,delta],
-                      [0,0,.5]]
+            points = [[0, 0, 0],
+                      [nu, nu, omega],
+                      [1 - nu, 1 - nu, 1 - omega],
+                      [nu, nu - 1, omega],
+                      [zeta, zeta, eta],
+                      [1 - zeta, -zeta, 1 - eta],
+                      [-zeta, -zeta, 1 - eta],
+                      [rho, 1 - rho, .5],
+                      [1 - rho, rho - 1, .5],
+                      [.5, .5, .5],
+                      [.5, 0, .5],
+                      [.5, 0, 0],
+                      [0, -.5, 0],
+                      [.5, -.5, 0],
+                      [mu, mu, delta],
+                      [1 - mu, -mu, -delta],
+                      [-mu, -mu, -delta],
+                      [mu, mu - 1, delta],
+                      [0, 0, .5]]
 
         return points
 
@@ -846,6 +848,8 @@ Angles of reciprocal lattice are kalpha={}, kbeta={}, kgamma={}.  \
 If you don't care, please use Cell.fromcellpar() instead."""
 
 # XXX labels, paths, are all the same.
+
+
 @bravaisclass('primitive triclinic', 'triclinic', 'triclinic', 'aP',
               ('a', 'b', 'c', 'alpha', 'beta', 'gamma'),
               [['TRI1a', 'GLMNRXYZ', 'XGY,LGZ,NGM,RG', None],
@@ -892,7 +896,7 @@ class TRI(BravaisLattice):
             if kgamma > min(kangles):
                 raise_unconventional()
             var = '1a'
-        elif all(kangles < 90):# and kgamma > max(kalpha, kbeta):
+        elif all(kangles < 90):  # and kgamma > max(kalpha, kbeta):
             if kgamma < max(kangles):
                 raise_unconventional()
             var = '1b'
@@ -905,23 +909,23 @@ class TRI(BravaisLattice):
         # (None of the points actually depend on any parameters)
         # (We should store the points openly on the variant objects)
         if variant.name == 'TRI1a' or variant.name == 'TRI2a':
-            points = [[0.,0.,0.],
-                      [.5,.5,0],
-                      [0,.5,.5],
-                      [.5,0,.5],
-                      [.5,.5,.5],
-                      [.5,0,0],
-                      [0,.5,0],
-                      [0,0,.5]]
+            points = [[0., 0., 0.],
+                      [.5, .5, 0],
+                      [0, .5, .5],
+                      [.5, 0, .5],
+                      [.5, .5, .5],
+                      [.5, 0, 0],
+                      [0, .5, 0],
+                      [0, 0, .5]]
         else:
-            points = [[0,0,0],
-                      [.5,-.5,0],
-                      [0,0,.5],
-                      [-.5,-.5,.5],
-                      [0,-.5,.5],
-                      [0,-0.5,0],
-                      [.5,0,0],
-                      [-.5,0,.5]]
+            points = [[0, 0, 0],
+                      [.5, -.5, 0],
+                      [0, 0, .5],
+                      [-.5, -.5, .5],
+                      [0, -.5, .5],
+                      [0, -0.5, 0],
+                      [.5, 0, 0],
+                      [-.5, 0, .5]]
 
         return points
 
@@ -1131,7 +1135,7 @@ def identify_lattice(cell, eps=2e-4, *, pbc=True):
         raise ValueError('System must be periodic either '
                          'along all three axes, '
                          'along two first axes or, '
-                         'along the thrid axis.  '
+                         'along the third axis.  '
                          'Got pbc={}'.format(pbc))
 
     from ase.geometry.bravais_type_engine import niggli_op_table
@@ -1341,6 +1345,17 @@ class LatticeChecker:
         mclc_cosa = 2.0 * prods[3] / (mclc_b * C)
         if -1 < mclc_cosa < 1:
             mclc_alpha = np.arccos(mclc_cosa) * 180 / np.pi
+            if mclc_b > C:
+                # XXX Temporary fix for certain otherwise
+                # unrecognizable lattices.
+                #
+                # This error could happen if the input lattice maps to
+                # something just outside the domain of conventional
+                # lattices (less than the tolerance).  Our solution is to
+                # propose a nearby conventional lattice instead, which
+                # will then be accepted if it's close enough.
+                mclc_b = 0.5 * (mclc_b + C)
+                C = mclc_b
             return self._check(MCLC, mclc_a, mclc_b, C, mclc_alpha)
 
     def TRI(self):
@@ -1408,7 +1423,7 @@ def get_2d_bravais_lattice(origcell, eps=2e-4, *, pbc=True):
                 lat = OBL(a, b, gamma)
                 rank = 1
 
-        op = lat.get_transformation(origcell)
+        op = lat.get_transformation(origcell, eps=eps)
         if not allclose(np.dot(op, lat.tocell())[pbc][:, pbc],
                         origcell.array[pbc][:, pbc]):
             msg = ('Cannot recognize cell at all somehow! {}, {}, {}'.
