@@ -1,15 +1,14 @@
-# flake8: noqa
 """Bravais.py - class for generating Bravais lattices etc.
 
 This is a base class for numerous classes setting up pieces of crystal.
 """
 
 import math
+from typing import Optional, Sequence
 
 import numpy as np
 
 from ase.atoms import Atoms
-from ase.utils import gcd, basestring
 import ase.data
 
 
@@ -29,11 +28,11 @@ class Bravais:
     #   int_basis
     #   inverse_basis
 
-    other = {0:(1,2), 1:(2,0), 2:(0,1)}
+    other = {0: (1, 2), 1: (2, 0), 2: (0, 1)}
 
     # For Bravais lattices with a basis, set the basis here.  Leave as
     # None if no basis is present.
-    bravais_basis = None
+    bravais_basis: Optional[Sequence[Sequence[float]]] = None
 
     # If more than one type of element appear in the crystal, give the
     # order here.  For example, if two elements appear in a 3:1 ratio,
@@ -41,14 +40,14 @@ class Bravais:
     # could be (0,0,1,0) - the third atom in the basis is different
     # from the other three.  Leave as None if all atoms are of the
     # same type.
-    element_basis = None
+    element_basis: Optional[Sequence[int]] = None
 
     # How small numbers should be considered zero in the unit cell?
     chop_tolerance = 1e-10
 
     def __call__(self, symbol,
-                 directions=(None,None,None), miller=(None,None,None),
-                 size=(1,1,1), latticeconstant=None,
+                 directions=(None, None, None), miller=(None, None, None),
+                 size=(1, 1, 1), latticeconstant=None,
                  pbc=True, align=True, debug=0):
         "Create a lattice."
         self.size = size
@@ -65,11 +64,14 @@ class Bravais:
             if self.element_basis is None:
                 self.latticeconstant = self.get_lattice_constant()
             else:
-                raise ValueError("A lattice constant must be specified for a compound")
+                raise ValueError(
+                    "A lattice constant must be specified for a compound")
         else:
             self.latticeconstant = latticeconstant
         if self.debug:
-            print("Expected number of atoms in unit cell:", self.calc_num_atoms())
+            print(
+                "Expected number of atoms in unit cell:",
+                self.calc_num_atoms())
         if self.debug >= 2:
             print("Bravais lattice basis:", self.bravais_basis)
             if self.bravais_basis is not None:
@@ -82,11 +84,12 @@ class Bravais:
 
     def align(self):
         "Align the first axis along x-axis and the second in the x-y plane."
-        degree = 180/np.pi
+        degree = 180 / np.pi
         if self.debug >= 2:
             print("Basis before alignment:")
             print(self.basis)
-        if self.basis[0][0]**2 + self.basis[0][2]**2 < 0.01 * self.basis[0][1]**2:
+        if self.basis[0][0]**2 + \
+                self.basis[0][2]**2 < 0.01 * self.basis[0][1]**2:
             # First basis vector along y axis - rotate 90 deg along z
             t = np.array([[0, -1, 0],
                           [1, 0, 0],
@@ -94,42 +97,43 @@ class Bravais:
             self.basis = np.dot(self.basis, t)
             transf = t
             if self.debug >= 2:
-                print("Rotating -90 degrees around z axis for numerical stability.")
+                print("Rotating -90 degrees around z axis for numerical "
+                      "stability.")
                 print(self.basis)
         else:
             transf = np.identity(3, np.float)
         assert abs(np.linalg.det(transf) - 1) < 1e-6
         # Rotate first basis vector into xy plane
-        theta = math.atan2(self.basis[0,2], self.basis[0,0])
+        theta = math.atan2(self.basis[0, 2], self.basis[0, 0])
         t = np.array([[np.cos(theta), 0, -np.sin(theta)],
-                      [         0, 1, 0          ],
-                      [np.sin(theta), 0, np.cos(theta) ]])
+                      [0, 1, 0],
+                      [np.sin(theta), 0, np.cos(theta)]])
         self.basis = np.dot(self.basis, t)
         transf = np.dot(transf, t)
         if self.debug >= 2:
-            print("Rotating %f degrees around y axis." % (-theta*degree,))
+            print("Rotating %f degrees around y axis." % (-theta * degree,))
             print(self.basis)
         assert abs(np.linalg.det(transf) - 1) < 1e-6
         # Rotate first basis vector to point along x axis
-        theta = math.atan2(self.basis[0,1], self.basis[0,0])
+        theta = math.atan2(self.basis[0, 1], self.basis[0, 0])
         t = np.array([[np.cos(theta), -np.sin(theta), 0],
-                      [np.sin(theta),  np.cos(theta), 0],
-                      [         0,           0, 1]])
+                      [np.sin(theta), np.cos(theta), 0],
+                      [0, 0, 1]])
         self.basis = np.dot(self.basis, t)
         transf = np.dot(transf, t)
         if self.debug >= 2:
-            print("Rotating %f degrees around z axis." % (-theta*degree,))
+            print("Rotating %f degrees around z axis." % (-theta * degree,))
             print(self.basis)
         assert abs(np.linalg.det(transf) - 1) < 1e-6
         # Rotate second basis vector into xy plane
-        theta = math.atan2(self.basis[1,2], self.basis[1,1])
+        theta = math.atan2(self.basis[1, 2], self.basis[1, 1])
         t = np.array([[1, 0, 0],
                       [0, np.cos(theta), -np.sin(theta)],
-                      [0, np.sin(theta),  np.cos(theta)]])
+                      [0, np.sin(theta), np.cos(theta)]])
         self.basis = np.dot(self.basis, t)
         transf = np.dot(transf, t)
         if self.debug >= 2:
-            print("Rotating %f degrees around x axis." % (-theta*degree,))
+            print("Rotating %f degrees around x axis." % (-theta * degree,))
             print(self.basis)
         assert abs(np.linalg.det(transf) - 1) < 1e-6
         # Now we better rotate the atoms as well
@@ -141,13 +145,14 @@ class Bravais:
         "Repeat the unit cell."
         nrep = self.size[0] * self.size[1] * self.size[2]
         if nrep <= 0:
-            raise ValueError("Cannot create a non-positive number of unit cells")
+            raise ValueError(
+                "Cannot create a non-positive number of unit cells")
         # Now the unit cells must be merged.
         a2 = []
         e2 = []
         for i in range(self.size[0]):
             offset = self.basis[0] * i
-            a2.append(self.atoms + offset[np.newaxis,:])
+            a2.append(self.atoms + offset[np.newaxis, :])
             e2.append(self.elements)
         atoms = np.concatenate(a2)
         elements = np.concatenate(e2)
@@ -155,7 +160,7 @@ class Bravais:
         e2 = []
         for j in range(self.size[1]):
             offset = self.basis[1] * j
-            a2.append(atoms + offset[np.newaxis,:])
+            a2.append(atoms + offset[np.newaxis, :])
             e2.append(elements)
         atoms = np.concatenate(a2)
         elements = np.concatenate(e2)
@@ -163,15 +168,15 @@ class Bravais:
         e2 = []
         for k in range(self.size[2]):
             offset = self.basis[2] * k
-            a2.append(atoms + offset[np.newaxis,:])
+            a2.append(atoms + offset[np.newaxis, :])
             e2.append(elements)
         atoms = np.concatenate(a2)
         elements = np.concatenate(e2)
         del a2, e2
         assert len(atoms) == nrep * len(self.atoms)
-        basis = np.array([[self.size[0],0,0],
-                          [0,self.size[1],0],
-                          [0,0,self.size[2]]])
+        basis = np.array([[self.size[0], 0, 0],
+                          [0, self.size[1], 0],
+                          [0, 0, self.size[2]]])
         basis = np.dot(basis, self.basis)
 
         # Tiny elements should be replaced by zero.  The cutoff is
@@ -191,12 +196,13 @@ class Bravais:
         "Extract atomic number from element"
         # The types that can be elements: integers and strings
         if self.element_basis is None:
-            if isinstance(element, basestring):
+            if isinstance(element, str):
                 self.atomicnumber = ase.data.atomic_numbers[element]
             elif isinstance(element, int):
                 self.atomicnumber = element
             else:
-                raise TypeError("The symbol argument must be a string or an atomic number.")
+                raise TypeError(
+                    "The symbol argument must be a string or an atomic number.")
         else:
             atomicnumber = []
             try:
@@ -209,15 +215,17 @@ class Bravais:
             if oops:
                 raise TypeError(
                     ("The symbol argument must be a sequence of length %d"
-                         +" (one for each kind of lattice position")
-                        % (max(self.element_basis)+1,))
+                     + " (one for each kind of lattice position")
+                    % (max(self.element_basis) + 1,))
             for e in element:
-                if isinstance(e, basestring):
+                if isinstance(e, str):
                     atomicnumber.append(ase.data.atomic_numbers[e])
                 elif isinstance(e, int):
                     atomicnumber.append(e)
                 else:
-                    raise TypeError("The symbols argument must be a sequence of strings or atomic numbers.")
+                    raise TypeError(
+                        "The symbols argument must be a sequence of strings "
+                        "or atomic numbers.")
             self.atomicnumber = [atomicnumber[i] for i in self.element_basis]
             assert len(self.atomicnumber) == len(self.bravais_basis)
 
@@ -226,14 +234,14 @@ class Bravais:
         self.directions = np.dot(self.directions, self.inverse_basis)
         if self.bravais_basis is not None:
             self.natural_bravais_basis = np.dot(self.bravais_basis,
-                                                        self.inverse_basis)
-        for i in (0,1,2):
+                                                self.inverse_basis)
+        for i in (0, 1, 2):
             self.directions[i] = reduceindex(self.directions[i])
-        for i in (0,1,2):
-            (j,k) = self.other[i]
+        for i in (0, 1, 2):
+            (j, k) = self.other[i]
             self.miller[i] = reduceindex(self.handedness *
                                          cross(self.directions[j],
-                                                self.directions[k]))
+                                               self.directions[k]))
 
     def calc_num_atoms(self):
         v = int(round(abs(np.linalg.det(self.directions))))
@@ -249,59 +257,58 @@ class Bravais:
         # in the real unit cell by put_atom().
         self.natoms = self.calc_num_atoms()
         self.nput = 0
-        self.atoms = np.zeros((self.natoms,3), np.float)
+        self.atoms = np.zeros((self.natoms, 3), np.float)
         self.elements = np.zeros(self.natoms, np.int)
         self.farpoint = sum(self.directions)
         # printprogress = self.debug and (len(self.atoms) > 250)
         # Find the radius of the sphere containing the whole system
         sqrad = 0
-        for i in (0,1):
-            for j in (0,1):
-                for k in (0,1):
+        for i in (0, 1):
+            for j in (0, 1):
+                for k in (0, 1):
                     vect = (i * self.directions[0] +
                             j * self.directions[1] +
                             k * self.directions[2])
-                    if np.dot(vect,vect) > sqrad:
-                        sqrad = np.dot(vect,vect)
-        del i,j,k
+                    if np.dot(vect, vect) > sqrad:
+                        sqrad = np.dot(vect, vect)
+        del i, j, k
         # Loop along first crystal axis (i)
-        for (istart, istep) in ((0,1), (-1,-1)):
+        for (istart, istep) in ((0, 1), (-1, -1)):
             i = istart
             icont = True
             while icont:
                 nj = 0
-                for (jstart, jstep) in ((0,1), (-1,-1)):
+                for (jstart, jstep) in ((0, 1), (-1, -1)):
                     j = jstart
                     jcont = True
                     while jcont:
                         nk = 0
-                        for (kstart, kstep) in ((0,1), (-1,-1)):
+                        for (kstart, kstep) in ((0, 1), (-1, -1)):
                             k = kstart
-                            #print "Starting line i=%d, j=%d, k=%d, step=(%d,%d,%d)" % (i,j,k,istep,jstep,kstep)
                             kcont = True
                             while kcont:
                                 # Now (i,j,k) loops over Z^3, except that
                                 # the loops can be cut off when we get outside
                                 # the unit cell.
-                                point = np.array((i,j,k))
+                                point = np.array((i, j, k))
                                 if self.inside(point):
                                     self.put_atom(point)
                                     nk += 1
                                     nj += 1
                                 # Is k too high?
-                                if np.dot(point,point) > sqrad:
+                                if np.dot(point, point) > sqrad:
                                     assert not self.inside(point)
                                     kcont = False
                                 k += kstep
                         # Is j too high?
-                        if i*i+j*j > sqrad:
+                        if i * i + j * j > sqrad:
                             jcont = False
                         j += jstep
                 # Is i too high?
-                if i*i > sqrad:
+                if i * i > sqrad:
                     icont = False
                 i += istep
-                #if printprogress:
+                # if printprogress:
                 #    perce = int(100*self.nput / len(self.atoms))
                 #    if perce > percent + 10:
                 #        print ("%d%%" % perce),
@@ -348,10 +355,10 @@ class Bravais:
         Find missing directions and miller indices from the specified ones.
         """
         directions = np.asarray(directions).tolist()
-        miller = np.asarray(miller).tolist()
+        miller = list(miller)  # np.asarray(miller).tolist()
         # If no directions etc are specified, use a sensible default.
         if directions == [None, None, None] and miller == [None, None, None]:
-            directions = [[1,0,0], [0,1,0], [0,0,1]]
+            directions = [[1, 0, 0], [0, 1, 0], [0, 0, 1]]
         # Now fill in missing directions and miller indices.  This is an
         # iterative process.
         change = 1
@@ -367,7 +374,9 @@ class Bravais:
                                                           miller[k]))
                         change = True
                         if self.debug >= 2:
-                            print("Calculating directions[%d] from miller indices" % i)
+                            print(
+                                "Calculating directions[%d] from miller "
+                                "indices" % i)
                 if miller[i] is None:
                     missing += 1
                     if directions[j] is not None and directions[k] is not None:
@@ -377,13 +386,16 @@ class Bravais:
                         if self.debug >= 2:
                             print("Calculating miller[%d] from directions" % i)
         if missing:
-            raise ValueError("Specification of directions and miller indices is incomplete.")
+            raise ValueError(
+                "Specification of directions and miller indices is incomplete.")
         # Make sure that everything is Numeric arrays
         self.directions = np.array(directions)
         self.miller = np.array(miller)
         # Check for zero-volume unit cell
         if abs(np.linalg.det(self.directions)) < 1e-10:
-            raise ValueError("The direction vectors are linearly dependent (unit cell volume would be zero)")
+            raise ValueError(
+                "The direction vectors are linearly dependent "
+                "(unit cell volume would be zero)")
         # Check for left-handed coordinate system
         if np.linalg.det(self.directions) < 0:
             print("WARNING: Creating a left-handed coordinate system!")
@@ -392,17 +404,21 @@ class Bravais:
         else:
             self.handedness = 1
         # Now check for consistency
-        for i in (0,1,2):
-            (j,k) = self.other[i]
+        for i in (0, 1, 2):
+            (j, k) = self.other[i]
             m = reduceindex(self.handedness *
                             cross(self.directions[j], self.directions[k]))
             if sum(np.not_equal(m, self.miller[i])):
-                print("ERROR: Miller index %s is inconsisten with directions %d and %d" % (i,j,k))
+                print(
+                    "ERROR: Miller index %s is inconsisten with "
+                    "directions %d and %d" % (i, j, k))
                 print("Miller indices:")
                 print(str(self.miller))
                 print("Directions:")
                 print(str(self.directions))
-                raise ValueError("Inconsistent specification of miller indices and directions.")
+                raise ValueError(
+                    "Inconsistent specification of miller indices and "
+                    "directions.")
 
     def print_directions_and_miller(self, txt=""):
         "Print direction vectors and Miller indices."
@@ -416,6 +432,7 @@ class Bravais:
 
 class MillerInfo:
     """Mixin class to provide information about Miller indices."""
+
     def miller_to_direction(self, miller):
         """Returns the direction corresponding to a given Miller index."""
         return np.dot(miller, self.millerbasis)
@@ -442,16 +459,16 @@ def cross(a, b):
 def reduceindex(M):
     """Reduce Miller index to the lowest equivalent integers."""
     oldM = M
-    g = gcd(M[0], M[1])
-    h = gcd(g, M[2])
+    g = math.gcd(M[0], M[1])
+    h = math.gcd(g, M[2])
     while h != 1:
         if h == 0:
-            raise ValueError("Division by zero: Are the miller indices linearly dependent?")
+            raise ValueError(
+                "Division by zero: Are the miller indices linearly dependent?")
         M = M // h
-        g = gcd(M[0], M[1])
-        h = gcd(g, M[2])
+        g = math.gcd(M[0], M[1])
+        h = math.gcd(g, M[2])
     if np.dot(oldM, M) > 0:
         return M
     else:
         return -M
-
