@@ -1,7 +1,10 @@
 import argparse
 import traceback
+from math import pi
 from time import time
 
+import numpy as np
+
 import ase.db
 import ase.optimize
 from ase.calculators.emt import EMT
@@ -14,6 +17,7 @@ all_optimizers.remove('QuasiNewton')
 
 
 def get_optimizer(name):
+    # types: (str) -> ase.optimize.Optimizer
     if name.startswith('Precon'):
         import ase.optimize.precon as precon
         return getattr(precon, name)
@@ -24,18 +28,37 @@ def get_optimizer(name):
 
 
 class Wrapper:
-    def __init__(self, atoms):
+    """Atoms-object wrapper that can count number of moves."""
+    def __init__(self, atoms, gridspacing=0.2, eggbox=0.0):
+        # types: (Atoms, float, float) -> None
         self.t0 = time()
         self.texcl = 0.0
         self.nsteps = 0
         self.atoms = atoms
         self.ready = False
-        self.pos = None
-        self.numbers = atoms.numbers
+        self.pos = None  # type: np.ndarray
+        self.eggbox = eggbox
+
+        if eggbox:
+            # Find small unit cell for grid-points
+            h = []
+            for axis in atoms.get_cell(complete=True):
+                L = np.linalg.norm(axis)
+                n = int(L / gridspacing)
+                h.append(axis / n)
+            self.x = np.linalg.inv(h)
+        else:
+            self.x = None
 
     def get_potential_energy(self, force_consistent=False):
         t1 = time()
         e = self.atoms.get_potential_energy(force_consistent)
+
+        if self.eggbox:
+            # Add egg-box error:
+            s = np.dot(self.atoms.positions, self.x)
+            e += np.cos(2 * pi * s).sum() * self.eggbox / 6
+
         t2 = time()
         self.texcl += t2 - t1
         if not self.ready:
@@ -46,6 +69,13 @@ class Wrapper:
     def get_forces(self):
         t1 = time()
         f = self.atoms.get_forces()
+
+        if self.eggbox:
+            # Add egg-box error:
+            s = np.dot(self.atoms.positions, self.x)
+            f += np.dot(np.sin(2 * pi * s),
+                        self.x.T) * (2 * pi * self.eggbox / 6)
+
         t2 = time()
         self.texcl += t2 - t1
         if not self.ready:
@@ -65,37 +95,19 @@ class Wrapper:
     def get_positions(self):
         return self.atoms.get_positions()
 
-    @property
-    def cell(self):
-        return self.atoms.cell
-
-    def get_cell(self, complete=False):
-        return self.atoms.get_cell(complete)
-
-    @property
-    def pbc(self):
-        return self.atoms.pbc
-
-    @property
-    def positions(self):
-        return self.atoms.positions
-
-    @property
-    def constraints(self):
-        return self.atoms.constraints
-
-    def copy(self):
-        return self.atoms.copy()
-
     def get_calculator(self):
         return self.atoms.calc
 
     def __len__(self):
         return len(self.atoms)
 
+    def __getattr__(self, name):
+        return self.atoms.__getattribute__(name)
 
-def run_test(atoms, optimizer, tag, fmax=0.02):
-    wrapper = Wrapper(atoms)
+
+def run_test(atoms, optimizer, tag, fmax=0.02, eggbox=0.0):
+    """Optimize atoms with optimizer."""
+    wrapper = Wrapper(atoms, eggbox=eggbox)
     relax = optimizer(wrapper, logfile=tag + '.log')
     relax.attach(Trajectory(tag + '.traj', 'w', atoms=atoms))
 
@@ -117,7 +129,10 @@ def run_test(atoms, optimizer, tag, fmax=0.02):
     return error, wrapper.nsteps, wrapper.texcl, tincl
 
 
-def test_optimizer(systems, optimizer, calculator, prefix='', db=None):
+def test_optimizer(systems, optimizer, calculator, prefix='', db=None,
+                   eggbox=0.0):
+    """Test optimizer on systems."""
+
     for name, atoms in systems:
         if db is not None:
             optname = optimizer.__name__
@@ -127,7 +142,8 @@ def test_optimizer(systems, optimizer, calculator, prefix='', db=None):
         atoms = atoms.copy()
         tag = '{}{}-{}'.format(prefix, optname, name)
         atoms.calc = calculator(txt=tag + '.txt')
-        error, nsteps, texcl, tincl = run_test(atoms, optimizer, tag)
+        error, nsteps, texcl, tincl = run_test(atoms, optimizer, tag,
+                                               eggbox=eggbox)
 
         if db is not None:
             db.write(atoms,
@@ -137,16 +153,21 @@ def test_optimizer(systems, optimizer, calculator, prefix='', db=None):
                      error=error,
                      n=nsteps,
                      t=texcl,
-                     T=tincl)
+                     T=tincl,
+                     eggbox=eggbox)
 
 
 def main():
     parser = argparse.ArgumentParser(
         description='Test ASE optimizers')
 
-    parser.add_argument('systems')
+    parser.add_argument('systems', help='File containing test systems.')
     parser.add_argument('optimizer', nargs='*',
-                        help='Optimizer name.')
+                        help='Optimizer name(s).  Choose from: {}. '
+                        .format(', '.join(all_optimizers)) +
+                        'Default is all optimizers.')
+    parser.add_argument('-e', '--egg-box', type=float, default=0.0,
+                        help='Fake egg-box error in eV.')
 
     args = parser.parse_args()
 
@@ -161,7 +182,7 @@ def main():
     for opt in args.optimizer:
         print(opt)
         optimizer = get_optimizer(opt)
-        test_optimizer(systems, optimizer, EMT, db=db)
+        test_optimizer(systems, optimizer, EMT, db=db, eggbox=args.egg_box)
 
 
 if __name__ == '__main__':
