@@ -14,7 +14,6 @@ import ase
 from ase.symbols import string2symbols
 from ase.spacegroup import Spacegroup
 from ase.geometry import cellpar_to_cell
-from ase.utils import basestring
 
 __all__ = ['crystal']
 
@@ -77,7 +76,7 @@ def crystal(symbols=None, basis=None, occupancies=None, spacegroup=1, setting=1,
         False, 0, 1, (1, 1, 0), (True, False, False).  Default
         is True.
     primitive_cell : bool
-        Wheter to return the primitive instead of the conventional
+        Whether to return the primitive instead of the conventional
         unit cell.
 
     Keyword arguments:
@@ -103,7 +102,7 @@ def crystal(symbols=None, basis=None, occupancies=None, spacegroup=1, setting=1,
     32
     """
     sg = Spacegroup(spacegroup, setting)
-    if (not isinstance(symbols, basestring) and
+    if (not isinstance(symbols, str) and
         hasattr(symbols, '__getitem__') and
         len(symbols) > 0 and
         isinstance(symbols[0], ase.Atom)):
@@ -165,8 +164,7 @@ def crystal(symbols=None, basis=None, occupancies=None, spacegroup=1, setting=1,
         info['unit_cell'] = 'primitive'
     else:
         info['unit_cell'] = 'conventional'
-
-
+        
     if 'info' in kwargs:
         info.update(kwargs['info'])
 
@@ -182,19 +180,15 @@ def crystal(symbols=None, basis=None, occupancies=None, spacegroup=1, setting=1,
                       masses=masses,
                       **kwargs)
 
-    #  if all occupancies are 1, no partial occupancy present
-    if occupancies:
-        if not all([occ == 1 for occ in occupancies]):
-            # use tags to identify sites, and in particular the occupancy
-            atoms.set_tags(kinds)
-
-
     if isinstance(basis, ase.Atoms):
         for name in basis.arrays:
             if not atoms.has(name):
                 array = basis.get_array(name)
                 atoms.new_array(name, [array[i] for i in kinds],
                                 dtype=array.dtype, shape=array.shape[1:])
+                
+    if kinds:
+        atoms.new_array('spacegroup_kinds', np.asarray(kinds, dtype=int))
 
     if primitive_cell:
         from ase.build import cut
@@ -212,6 +206,6 @@ def crystal(symbols=None, basis=None, occupancies=None, spacegroup=1, setting=1,
 
 def parse_symbols(symbols):
     """Return `sumbols` as a sequence of element symbols."""
-    if isinstance(symbols, basestring):
+    if isinstance(symbols, str):
         symbols = string2symbols(symbols)
     return symbols
