@@ -21,8 +21,6 @@ def test_cartesian(atoms=atoms):
 
     assert np.allclose(atoms.positions, new_atoms.positions)
 
-    Path(file).unlink()
-
 
 # check scaled
 def test_scaled(atoms=atoms):
@@ -36,8 +34,6 @@ def test_scaled(atoms=atoms):
         new_atoms.positions,
     )
 
-    Path(file).unlink()
-
 
 # this should fail
 def test_scaled_wrapped(atoms=atoms):
@@ -57,11 +53,3 @@ def test_scaled_wrapped(atoms=atoms):
             atoms.positions,
             new_atoms.positions,
         )
-
-    Path(file).unlink()
-
-
-if __name__ == "__main__":
-    test_cartesian()
-    test_scaled()
-    test_scaled_wrapped()
