@@ -36,6 +36,8 @@ forward_methods = ['__abs__', '__add__', '__contains__', '__eq__',
                    '__rsub__', '__rtruediv__', '__setitem__',
                    '__sub__', '__truediv__']
 
+default_methods = ['__eq__', '__le__', '__lt__', '__ge__',
+                   '__gt__', '__ne__', '__hash__']
 
 if hasattr(np.ndarray, '__matmul__'):
     forward_methods += ['__matmul__', '__rmatmul__']
@@ -43,6 +45,7 @@ if hasattr(np.ndarray, '__matmul__'):
 
 def forward_inplace_call(name):
     arraymeth = getattr(np.ndarray, name)
+
     def f(self, obj):
         a = self.__array__()
         arraymeth(a, obj)
@@ -58,11 +61,12 @@ def wrap_array_attribute(name):
     if wrappee is None:  # For example, __hash__ is None
         assert name == '__hash__'
         return None
+
     def attr(self):
         array = np.asarray(self)
         return getattr(array, name)
     attr.__name__ = wrappee.__name__
-    attr.__qualname__ == wrappee.__qualname__
+    attr.__qualname__ = wrappee.__qualname__
     return property(attr)
 
 
@@ -78,11 +82,11 @@ def arraylike(cls):
     for name in inplace_methods:
         if hasattr(np.ndarray, name) and not hasattr(cls, name):
             meth = forward_inplace_call(name)
-        setattr(cls, name, meth)
+            setattr(cls, name, meth)
 
     allnames = [name for name in dir(np.ndarray) if not name.startswith('_')]
     for name in forward_methods + allnames:
-        if hasattr(cls, name) and not name.startswith('_'):
+        if hasattr(cls, name) and name not in default_methods:
             continue  # Was overridden -- or there's a conflict.
 
         prop = wrap_array_attribute(name)
