@@ -7,10 +7,7 @@ import sys
 def output_to_string(pythonfile):
     """Returns the stdout of executing the code in pythonfile
     as a string."""
-    if sys.version_info.major == 2:
-        buffer = io.BytesIO()
-    else:
-        buffer = io.StringIO()
+    buffer = io.StringIO()
     sys.stdout = buffer
     exec(open(pythonfile).read())
     sys.stdout = sys.__stdout__
