@@ -13,7 +13,7 @@ eps = 0.01
 for a in a0 * np.linspace(1 - eps, 1 + eps, 3):
     for c in c0 * np.linspace(1 - eps, 1 + eps, 3):
         ni = bulk('Ni', 'hcp', a=a, c=c)
-        ni.set_calculator(EMT())
+        ni.calc = EMT()
         ni.get_potential_energy()
         traj.write(ni)
 
